from __future__ import division
import numpy as np
import sympy as sp

class GNDclass(object):
    def __repr__(self):
        return "GND"
GND = GNDclass()

class NODESclass(object):
    def add_count(self, tc, conn, param): pass
    def process(self, p, conn, param, alpha): pass
    def __repr__(self): return "NODES"
NODES = NODESclass()

class Node(object):
    def __init__(self, nm, d):
        self.nm = nm
        self.d = d
    def __repr__(self):
        return "%s%s" % (self.nm, self.d or "")
    def __hash__(self):
        return hash((self.nm,self.d))
    def __eq__(self, o):
        if not isinstance(o, Node):
            return False
        return self.nm == o.nm and self.d == o.d
    def __call__(self, s, fact=1.0):
        return OutU(self, s, fact)
    def check(self, param):
        if param is None:
            raise ValueError("%s: no value defined" % self)

class INclass(Node):
    def __init__(self):
        Node.__init__(self, "IN", None)
    def add_count(self, tc, conn, param):
        self.conn = conn
        tc["V"] += len(conn)
        tc["I"] = len(conn)
    def process(self, p, conn, param, alpha):
        for i, c in enumerate(conn):
            idx = p.new_row("V", self, self.conn[i])
            p.S[idx, c] += 1
            p.S[c, idx] += 1
            n = p.new_row("I", self, self.conn[i])
            p.N["I"][n, idx] = 1
IN = INclass()


class Out(object):
    def __init__(self, node, fact=1.0):
        self.node = node
        self.fact = fact

    def get_index(self, p):
        return p.nodes[self.node], self.fact

    def __repr__(self):
        if self.fact != 1.0:
            return "%s*%g" % (self.node, self.fact)
        else:
            return repr(self.node)


class OutU(Out):
    def __init__(self, node, f, fact):
        self.node = node
        self.f = f
        self.fact = fact

    def get_index(self, p):
        return p.extra_variable_by_name((self.node,self.f)), self.fact

    def __repr__(self):
        return "%s%s" % (self.node, self.f and ("[%s]" % self.f) or "")

class OUTclass(Node):
    def __init__(self):
        Node.__init__(self, "OUT", None)
    def add_count(self, tc, conn, param):
        self.conn = conn
        tc["O"] += len(conn)
    def process(self, p, conn, param, alpha):
        for c in conn:
            v = 1
            if isinstance(c, Out):
                c, fact = c.get_index(p)
                v = fact
            p.N["O"][p.new_row("O", self), c] = v
OUT = OUTclass()

class R(Node):
    def __init__(self, n=None):
        Node.__init__(self, "R", n)
    def add_count(self, tc, conn, param):
        tc["R"] += 1
    def process(self, p, conn, param, alpha):
        idx = p.new_row("R", self)
        #param = sp.symbols(str(self))
        p.add_S_currents(conn, 1 / param)
        p.add_2conn("R", idx, conn)

class P_parallel(Node):
    # loga value     pot value for var = 0.5
    # log(16)        20%
    # 5              ~7.6%
    def __init__(self, n=None):
        Node.__init__(self, "P", n)
    def add_count(self, tc, conn, param):
        if conn[0] is not None and conn[1] is not None and conn[0] != conn[2] and conn[2] != conn[1]:
            tc["R"] += 2
            tc["P"] += 2
        else:
            tc["R"] += 1
            tc["P"] += 1
        tc["v"] += 1
    def process(self, p, conn, param, alpha):
        self.check(param)
        sym = str(self)+"v"
        if isinstance(param, dict):
            val = 2 * param["value"]
            sym = param.get("var", sym)
        else:
            val = 2 * param
        a = sp.symbols(sym)
        v = 1 / val
        # first resistor
        if conn[0] != conn[2] and conn[0] is not None:
            c = (conn[0], conn[2])
            idx1 = p.new_row("R", self)
            p.add_S_currents(c, v)
            p.add_2conn("R", idx1, c)
            idx_p1 = p.new_row("P", self, "+")
            p.add_2conn("P", idx_p1, c)
            p.Pv[idx_p1] = val
            p.pot_func[idx_p1] = (a, a / (2 - a))
        if conn[2] != conn[1] and conn[1] is not None:
            # second resistor
            c = (conn[2], conn[1])
            idx2 = p.new_row("R", self)
            p.add_S_currents(c, v)
            p.add_2conn("R", idx2, c)
            idx_p2 = p.new_row("P", self, "-")
            p.add_2conn("P", idx_p2, c)
            p.Pv[idx_p2] = val
            p.pot_func[idx_p2] = (a, (1 - a) / (1 + a))


class P_single(Node):
    def __init__(self, n=None):
        Node.__init__(self, "P", n)
    def add_count(self, tc, conn, param):
        if len(conn) == 3:
            tc["P"] += 2
        else:
            tc["P"] += 1
        tc["v"] += 1
    def process(self, p, conn, param, alpha):
        sym = str(self)+"v"
        if isinstance(param, dict):
            val = param["value"]
            sym = param.get("var", sym)
        else:
            val = param
        a = sp.symbols(sym)
        # first resistor
        c = (conn[0], conn[2])
        idx_p1 = p.new_row("P", self, "+")
        p.add_2conn("P", idx_p1, c)
        p.Pv[idx_p1] = val
        p.pot_func[idx_p1] = (a, a)
        if len(conn) == 3:
            # second resistor
            c = (conn[1], conn[2])
            idx_p2 = p.new_row("P", self, "-")
            p.add_2conn("P", idx_p2, c)
            p.Pv[idx_p2] = val
            p.pot_func[idx_p2] = (a, 1 - a)

class P_fixed(Node):
    def __init__(self, n=None):
        Node.__init__(self, "P", n)
    def add_count(self, tc, conn, param):
        if len(conn) == 3:
            tc["R"] += 2
        else:
            tc["R"] += 1
    def process(self, p, conn, param, alpha):
        if isinstance(param, dict):
            val = param["value"]
        else:
            val = param
        # first resistor
        c = (conn[0], conn[2])
        idx_p1 = p.new_row("R", self, "+")
        p.add_S_currents(conn, 2 / val)
        p.add_2conn("R", idx_p1, c)
        if len(conn) == 3:
            # second resistor
            c = (conn[1], conn[2])
            idx_p2 = p.new_row("R", self, "-")
            p.add_S_currents(conn, 2 / val)
            p.add_2conn("R", idx_p2, c)

P = P_parallel
#P = P_single
#P = P_fixed

class C(Node):
    def __init__(self, n=None):
        Node.__init__(self, "C", n)
    def add_count(self, tc, conn, param):
        tc["X"] += 1
    def process(self, p, conn, param, alpha):
        #print ("C = %s %s" % (param, alpha))
        idx = p.new_row("X", self)
        #param = sp.symbols(str(self))
        value = alpha * param
        #print ("C value = %s " % value)
        p.add_S_currents(conn, value)
        p.add_2conn("Xl", idx, conn, value)
        p.add_2conn("Xr", idx, conn)
        p.Z[idx] = 1

class L(Node):
    def __init__(self, n=None):
        Node.__init__(self, "L", n)
    def add_count(self, tc, conn, param):
        tc["X"] += 1
    def process(self, p, conn, param, alpha):
        #print ("L = %s %s" % (param, alpha))
        idx = p.new_row("X", self)
        #param = sp.symbols(str(self))
        #value = 1 / (alpha*2 * param)
        value = 1 / (alpha * param)
        #print ("L value = %s " % value)
        p.add_S_currents(conn, value)
        p.add_2conn("Xl", idx, conn, value)
        p.add_2conn("Xr", idx, conn)
        p.Z[idx] = -1

class D(Node):
    def __init__(self, n=None):
        Node.__init__(self, "D", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 1
    def process(self, p, conn, param, alpha):
        Is, mUt = const = sp.symbols("Is,mUt")
        v0, = v = sp.symbols("v:1", seq=True)
        calc = -Is * (sp.exp(v0/mUt) - 1)
        calc = calc.subs(dict([(k,param[str(k)]) for k in const]))
        idx = p.new_row("N", self)
        p.add_2conn("Nl", idx, conn)
        p.add_2conn("Nr", idx, conn)
        p.set_function(idx, calc, v, idx)

class D2(Node):
    def __init__(self, n=None):
        Node.__init__(self, "D", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 1
    def process(self, p, conn, param, alpha):
        Is, mUt = const = sp.symbols("Is,mUt")
        v0, = v = sp.symbols("v:1", seq=True)
        calc = -2 * Is * sp.sinh(v0/mUt)
        calc = calc.subs(dict([(k,param[str(k)]) for k in const]))
        idx = p.new_row("N", self)
        p.add_2conn("Nl", idx, (conn[0], conn[1]))
        p.add_2conn("Nr", idx, (conn[0], conn[1]))
        p.set_function(idx, calc, v, idx)

class VCCS(Node):
    def __init__(self, n=None):
        Node.__init__(self, "VCCS", n)
    def add_count(self, tc, conn, param):
        pass
    def process(self, p, conn, param, alpha):
        dG = param["dG"]
        i0 = param["i0"]
        if conn[2] != -1:
            if conn[0] != -1:
                p.S[conn[2], conn[0]] += dG
            if conn[1] != -1:
                p.S[conn[2], conn[1]] -= dG
            p.ConstVoltages[0,conn[2]] += i0
        if conn[3] != -1:
            if conn[0] != -1:
                p.S[conn[3], conn[0]] -= dG
            if conn[1] != -1:
                p.S[conn[3], conn[1]] += dG
            p.ConstVoltages[0,conn[3]] -= i0

class T(Node):
    def __init__(self, n=None):
        Node.__init__(self, "T", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 2
    def process(self, p, conn, param, alpha):
        # pins are C, B, E (index 0, 1, 2)
        # Ib and Ie depend on Vbc (1 - 0) and Vbe (1 - 2), Ic = Ib + Ie
        # 
        Is, Bf, Vt, Br = const = sp.symbols("Is,Bf,Vt,Br")
        Vbc, Vbe = v = sp.symbols("Vbc,Vbe")
        calc_ib = -(Is / Bf * (sp.exp(Vbe/Vt)-1) + Is/Br * (sp.exp(Vbc/Vt)-1))
        calc_ib = calc_ib.subs(dict([(k,param[str(k)]) for k in const]))
        calc_ie = -(-Is*(sp.exp(Vbe/Vt)-1) + Is*(Br-1)/Br * (sp.exp(Vbc/Vt)-1))
        calc_ie = calc_ie.subs(dict([(k,param[str(k)]) for k in const]))
        idx1 = p.new_row("N", self, "Ib")
        p.add_2conn("Nl", idx1, (conn[1],conn[0]))
        p.add_2conn("Nr", idx1, (conn[1],conn[0]))
        p.set_function(idx1, calc_ib, v, idx1)
        idx2 = p.new_row("N", self, "Ie")
        p.add_2conn("Nl", idx2, (conn[1],conn[2]))
        p.add_2conn("Nr", idx2, (conn[2],conn[0]))
        p.set_function(idx2, calc_ie, v, idx1)

class Tp(Node):
    def __init__(self, n=None):
        Node.__init__(self, "T", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 2
    def process(self, p, conn, param, alpha):
        # pins are C, B, E (index 0, 1, 2)
        # Ib and Ie depend on Vbc (1 - 0) and Vbe (1 - 2), Ic = Ie - Ib
        # 
        Is, Bf, Vt, Br = const = sp.symbols("Is,Bf,Vt,Br")
        Vbc, Vbe = v = sp.symbols("Vbc,Vbe")
        calc_ib = -(Is / Bf * (sp.exp(Vbe/Vt)-1) + Is/Br * (sp.exp(Vbc/Vt)-1))
        calc_ib = calc_ib.subs(dict([(k,param[str(k)]) for k in const]))
        calc_ie = -(-Is*(sp.exp(Vbe/Vt)-1) + Is*(Br-1)/Br * (sp.exp(Vbc/Vt)-1))
        calc_ie = calc_ie.subs(dict([(k,param[str(k)]) for k in const]))
        idx1 = p.new_row("N", self, "Ib")
        p.add_2conn("Nl", idx1, (conn[0],conn[1]))
        p.add_2conn("Nr", idx1, (conn[0],conn[1]))
        p.set_function(idx1, calc_ib, v, idx1)
        idx2 = p.new_row("N", self, "Ie")
        p.add_2conn("Nl", idx2, (conn[2],conn[1]))
        p.add_2conn("Nr", idx2, (conn[0],conn[2]))
        p.set_function(idx2, calc_ie, v, idx1)

class Triode(Node):
    def __init__(self, n=None):
        Node.__init__(self, "Triode", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 2
    def process(self, p, conn, param, alpha):
        mu, Ex, Kp, Kvb, Kg1, Gco, Gcf = const = sp.symbols("mu,Ex,Kp,Kvb,Kg1,Gco,Gcf")
        Ugk, Uak = v = sp.symbols("Ugk,Uak", real=True) # "real" needed to get derivative of sign()
        t = Kp*(1/mu+Ugk/sp.sqrt(Kvb+Uak*Uak))
        E1 = Uak/Kp*sp.log(1+sp.exp(t))
        E1_ = Uak/Kp*t
        calc_Ia = sp.Piecewise(
            (0, Uak < 0),
            (0, t < -500),
            (-pow(E1_,Ex) / Kg1 * (1+sp.sign(E1_)), t > 500),
            (-pow(E1,Ex) / Kg1 * (1+sp.sign(E1)), True))
        calc_Ia = calc_Ia.subs(dict([(k,param[str(k)]) for k in const]))
        calc_Ig = sp.Piecewise((0, Ugk < Gco), (-Gcf*pow(Ugk-Gco, 1.5), True))
        calc_Ig = calc_Ig.subs(dict([(k,param[str(k)]) for k in const]))
        # def calc_Ia(v):
        #     Ugk = float(v[0])
        #     Uak = float(v[1])
        #     if Uak < 0:
        #         return 0
        #     t = Kp*(1/mu+Ugk/math.sqrt(Kvb+Uak*Uak))
        #     if t > 500:
        #         E1 = Uak/Kp*t
        #     elif t < -500:
        #         return 0
        #     else:
        #         E1 = Uak/Kp*math.log(1+math.exp(t))
        #     r = pow(E1,Ex) / Kg1 * 2*(E1 > 0.0)
        #     return -r
        # def calc_Ig(v):
        #     Ugk = float(v[0])
        #     if Ugk < Gco:
        #         return 0
        #     r = Gcf*pow(Ugk-Gco, 1.5)
        #     return -r
        idx1 = p.new_row("N", self, "Ig")
        p.add_2conn("Nl", idx1, (conn[0],conn[2]))
        p.add_2conn("Nr", idx1, (conn[0],conn[2]))
        p.set_function(idx1, calc_Ig, v, idx1)
        idx2 = p.new_row("N", self, "Ip")
        p.add_2conn("Nl", idx2, (conn[1],conn[2]))
        p.add_2conn("Nr", idx2, (conn[1],conn[2]))
        p.set_function(idx2, calc_Ia, v, idx1)

class Pentode(Node):
    def __init__(self, n=None):
        Node.__init__(self, "Pentode", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 3
    def process(self, p, conn, param, alpha):
        mu, Ex, Kp, Kg1, Kg2, Kvb, Gco, Gcf = const = sp.symbols("mu,Ex,Kp,Kg1,Kg2,Kvb,Gco,Gcf")
        Ug1k, Ug2k, Uak = v = sp.symbols("Ug1k,Ug2k,Uak")
        t = Kp * (1 / mu + Ug1k / Ug2k)
        E1 = Ug2k / Kp * sp.log(1 + sp.exp(t))
        E1_ = Ug2k / Kp * t
        #sign = sp.sign
        sign = lambda x: 2 * sp.Heaviside(x) - 1
        calc_Ia = sp.Piecewise(
            (0, Ug2k <= 0),
            (0, t < -500),
            (-pow(E1_,Ex)/Kg1 * (1+sign(E1_)) * sp.atan(Uak/Kvb), t > 500),
            (-pow(E1,Ex)/Kg1 * (1+sign(E1)) * sp.atan(Uak/Kvb), True))
        calc_Ia = calc_Ia.subs(dict([(k,param[str(k)]) for k in const]))
        calc_Ig = sp.Piecewise((0, Ug1k < Gco), (-Gcf*pow(Ug1k-Gco, 1.5), True))
        calc_Ig = calc_Ig.subs(dict([(k,param[str(k)]) for k in const]))
        t = Ug2k / mu + Ug1k
        calc_Is = sp.Piecewise((0, t <= 0), (-sp.exp(Ex*sp.log(t)) / Kg2, True))
        calc_Is = calc_Is.subs(dict([(k,param[str(k)]) for k in const]))
        # def calc_Ia(v):
        #     Ug1k = float(v[0])
        #     Ug2k = float(v[1])
        #     Uak = float(v[2])
        #     if Ug2k <= 0.0:
        #         return 0
        #     t = Kp * (1 / mu + Ug1k / Ug2k)
        #     if t > 500:
        #         E1 = Ug2k / Kp * t
        #     elif t < -500:
        #         return 0
        #     else:
        #         E1 = Ug2k / Kp * math.log(1 + math.exp(t))
        #     r = pow(E1,Ex)/Kg1 * 2*(E1 > 0.0) * math.atan(Uak/Kvb);
        #     #print Ug1k, Ug2k, Uak, r
        #     return -r
        # def calc_Ig(v):
        #     Ugk = float(v[0])
        #     if Ugk < Gco:
        #         return 0
        #     r = Gcf*pow(Ugk-Gco, 1.5)
        #     return -r
        # def calc_Is(v):
        #     Ug1k = float(v[0])
        #     Ug2k = float(v[1])
        #     t = Ug2k / mu + Ug1k
        #     if t <= 0:
        #         return 0
        #     r = math.exp(Ex*math.log(t)) / Kg2
        #     return -r

        idx1 = p.new_row("N", self, "Ig")
        p.add_2conn("Nl", idx1, (conn[0],conn[3]))
        p.add_2conn("Nr", idx1, (conn[0],conn[3]))
        p.set_function(idx1, calc_Ig, v[:1], idx1)
        idx2 = p.new_row("N", self, "Is")
        p.add_2conn("Nl", idx2, (conn[1],conn[3]))
        p.add_2conn("Nr", idx2, (conn[1],conn[3]))
        p.set_function(idx2, calc_Is, v, idx1)
        idx3 = p.new_row("N", self, "Ip")
        p.add_2conn("Nl", idx3, (conn[2],conn[3]))
        p.add_2conn("Nr", idx3, (conn[2],conn[3]))
        p.set_function(idx3, calc_Ia, v, idx1)

class Trans_L(Node):
    def __init__(self, n=None, nw=3):
        self.nw = nw
        Node.__init__(self, "TL", n)
    def add_count(self, tc, conn, param):
        tc["X"] += 1
        tc["V"] += self.nw + 2
    def process(self, p, conn, param, alpha):
        start = p.current_row("V") + 1
        end = start + self.nw + 1
        for i in range(self.nw):
            s = "W%d" % (i+1)
            idx = p.new_row("V", self, s)
            p.add_S(idx, conn[2*i:], 1)
            p.S[idx, end] += param["windings"][i]
        idx = p.new_row("V", self, "phi")
        p.S[idx, start:end-1] += np.array(param["windings"])
        p.S[idx, end-1] += param["R"]
        idx_v = p.new_row("V", self, "v")
        p.S[idx_v, idx_v] += 1
        p.S[idx_v, idx] += alpha
        idx_xv = p.new_row("X", self, "v")
        p.N["Xl"][idx_xv, idx] = alpha
        p.N["Xr"][idx_xv, idx_v] = 1
        p.Z[idx_xv] = 1

class Trans_F(Node):
    def __init__(self, n=None, nw=3):
        self.nw = nw
        Node.__init__(self, "TF", n)
    def add_count(self, tc, conn, param):
        tc["X"] += 1
        tc["V"] += self.nw + 2
        tc["N"] += 1
    def process(self, p, conn, param, alpha):
        b, c = const = sp.symbols("b,c")
        v0, = v = sp.symbols("v:1", seq=True)
        calc_frohlich = -(c * v0) / (1 - b * abs(v0))
        calc_frohlich = calc_frohlich.subs(dict([(k,param[str(k)]) for k in const]))
        # def calc_frohlich(v):
        #     v = float(v[0])
        #     #b = 255.
        #     #c = 358.
        #     r = (c * v) / (1 - b * abs(v))
        #     return -r
        start = p.current_row("V") + 1
        end = start + self.nw + 1
        for i in range(self.nw):
            s = "W%d" % (i+1)
            idx = p.new_row("V", self, s)
            p.add_S(idx, conn[2*i:], 1)
            p.S[idx, end] += param["windings"][i]
        idx = p.new_row("V", self, "phi")
        p.S[idx, start:end-1] += np.array(param["windings"])
        idx_v = p.new_row("V", self, "v")
        p.S[idx_v, idx_v] += 1
        p.S[idx_v, idx] += alpha
        idx_xv = p.new_row("X", self, "v")
        p.N["Xl"][idx_xv, idx] = alpha
        p.N["Xr"][idx_xv, idx_v] = 1
        p.Z[idx_xv] = 1
        idx = p.new_row("N", self, "phi")
        p.N["Nl"][idx, end-1] = 1
        p.N["Nr"][idx, end-1] = 1
        p.set_function(idx, calc_frohlich, v, idx)

class Trans_GC(Node):
    def __init__(self, n=None, nw=3):
        self.nw = nw
        Node.__init__(self, "TG", n)
    def add_count(self, tc, conn, param):
        tc["X"] += 1
        tc["V"] += self.nw + 2
        tc["N"] += 1
    def process(self, p, conn, param, alpha):
        windings = np.array(param["windings"], dtype=np.float64)
        if self.nw == 1:
            # avoid some convergence problems by scaling N to 1
            wind = windings[0]
            windings /= wind
            param = dict(C = param["C"] * wind**2,
                         a = param["a"] * wind**(param["n"]-1),
                         o = param["o"] * wind**-1,
                         n = param["n"],
                         )
        C, a, o, n = const = sp.symbols("C,a,o,n")
        phi, = v = sp.symbols("phi", seq=True, real=True) # "real" needed to get derivative of sign()
        t = phi / C
        calc_gc_MMF = -(a * pow(abs(t), n) * sp.sign(t)) + o
        calc_gc_MMF = calc_gc_MMF.subs(dict([(k,param[str(k)]) for k in const]))
        # def calc_gc_MMF(v):
        #     v = float(v[0])
        #     #C = 2e-3
        #     #a = 1e-5
        #     #n = 25
        #     t = v / C
        #     if v >= 0:
        #         t2 = pow(t, n)
        #     else:
        #         t2 = -pow(-t, n)
        #     r = -(t + a * t2)
        #     return r
        start = p.current_row("V") + 1
        end = start + self.nw + 1
        for i in range(self.nw):
            s = "W%d" % (i+1)
            idx = p.new_row("V", self, s)
            p.add_S(idx, conn[2*i:], 1)
            p.S[idx, end] += windings[i]
        idx = p.new_row("V", self, "phi")
        for i in range(self.nw):
            p.S[idx, start+i] += windings[i]
        p.S[idx, end-1] += 1 / param["C"]
        idx_v = p.new_row("V", self, "v")
        p.S[idx_v, idx_v] += 1
        p.S[idx_v, idx] += alpha
        idx_xv = p.new_row("X", self, "v")
        p.N["Xl"][idx_xv, idx] = alpha
        p.N["Xr"][idx_xv, idx_v] = 1
        p.Z[idx_xv] = 1
        idx = p.new_row("N", self, "phi")
        p.N["Nl"][idx, end-1] = 1
        p.N["Nr"][idx, end-1] = 1
        p.set_function(idx, calc_gc_MMF, v, idx)

class V(Node):
    def __init__(self, n=None):
        Node.__init__(self, "V", n)
    def add_count(self, tc, conn, param):
        tc["V"] += 1
        tc["C"] += 1
    def process(self, p, conn, param, alpha):
        idx = p.new_row("V", self)
        p.S[idx, conn[0]] += 1
        p.S[conn[0], idx] += 1
        p.ConstVoltages[0,idx] += param

class OPA(Node):
    def __init__(self, n=None):
        Node.__init__(self, "OPA_L", n)
    def add_count(self, tc, conn, param):
        tc["V"] += 1
        if isinstance(param, dict) and "Vcc" in param:
            tc["N"] += 1
    def process(self, p, conn, param, alpha):
        idx_s = p.new_row("V", self)
        p.S[idx_s, conn[2]] += 1
        p.S[conn[2], idx_s] += 1
        if isinstance(param, dict) and "Vcc" in param:
            Vcc, Vee, A = const = sp.symbols("Vcc,Vee,A")
            v0, = v = sp.symbols("v:1", seq=True)
            a = 2*A/(Vcc-Vee)
            calc = 0.5 * (sp.tanh(a*v0) * (Vcc-Vee) + Vcc + Vee)
            calc = calc.subs(dict([(k,param[str(k)]) for k in const]))
            idx = p.new_row("N", self)
            p.add_conn("Nl", idx, conn[0], 1)
            p.add_conn("Nl", idx, conn[1], -1)
            p.add_conn("Nr", idx, idx_s, 1)
            p.set_function(idx, calc, v, idx)
        else:
            if isinstance(param, dict):
                param = param["A"]
            p.S[idx_s, conn[0]] += param
            p.S[idx_s, conn[1]] += -param

class CC_L(Node):
    def __init__(self, n=None):
        Node.__init__(self, "CC", n)
    def add_count(self, tc, conn, param):
        tc["V"] += 1
    def process(self, p, conn, param, alpha):
        idx = p.new_row("V", self)
        p.S[conn[0], idx] += 1
        p.S[conn[1], idx] -= 1
        p.S[idx, conn[0]] += 1
        p.S[idx, conn[1]] -= 1

class CC_N(Node):
    def __init__(self, n=None):
        Node.__init__(self, "CC", n)
    def add_count(self, tc, conn, param):
        tc["N"] += 1
    def process(self, p, conn, param, alpha):
        v0, = v = sp.symbols("v:1", seq=True)
        calc = v0
        idx = p.new_row("N", self)
        p.add_2conn("Nl", idx, conn)
        p.add_2conn("Nr", idx, conn)
        p.set_function(idx, calc, v, idx)
        p.CZ[idx] = 0

#CC = CC_L
CC = CC_N



from __future__ import division
import numpy as np
import numpy.matlib as ml
import numpy.linalg as la
import scipy.linalg as sla
import scipy.optimize as opt
import scipy.signal as sig
import sympy as sp
import ctypes as ct
import slicot
import collections, warnings, tempfile, os, operator, sys, shutil, time
import commands, subprocess, math, re, logging

import dk_templates, generate_code
from models import GND, Out, Node
from dk_lib import printoptions

try:
    opt.root
except AttributeError:
    class RootResult(object):
        def __init__(self, x, infodict, ier, mesg):
            self.x = x
            self.success = (ier == 1)
            self.status = ier
            self.message = mesg
    def opt_root(fun, x0, args=(), method='hybr', jac=None, tol=None, callback=None, options=None):
        factor = 100
        if options:
            factor = options.get("factor", factor)
        return RootResult(*opt.fsolve(fun, x0, args, jac, full_output=True))
    opt.root = opt_root

################################################################
# bugfixes
#
from sympy.printing.ccode import CCodePrinter

def _print_Piecewise(self, expr):
    # This method is called only for inline if constructs
    # Top level piecewise is handled in doprint()
    ecpairs = ["(%s) ? (%s) : " % (self._print(c), self._print(e)) \
                   for e, c in expr.args[:-1]]
    last_line = ""
    if expr.args[-1].cond == True:
        last_line = "(%s)" % self._print(expr.args[-1].expr)
    else:
        ecpairs.append("(%s) ? (%s)" % \
                       (self._print(expr.args[-1].cond),
                        self._print(expr.args[-1].expr)))
    return "(" + "".join(ecpairs) + last_line + ")"

CCodePrinter._print_Piecewise = _print_Piecewise
#
# end bugfixes
################################################################

def pkgconfig(*packages, **kw):
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}
    for token in commands.getoutput("pkg-config --libs --cflags %s" % ' '.join(packages)).split():
        kw.setdefault(flag_map.get(token[:2]), []).append(token[2:])
    return kw

def matrix_add(*args):
    l = [m for m in args if m.size]
    if l:
        return reduce(ml.add, l)
    else:
        return args[0]


class TransformOptions(object):

    def __init__(self, **kw):
        self._set(**kw)

    def _set(self, partition=False, chain=False, svd_prec=None, decompose=False):
        self.partition = partition
        self.chain = chain
        self.svd_prec = svd_prec
        self.decompose = decompose

    def copy(self, **kw):
        t = TransformOptions()
        t.__dict__.update(self.__dict__)
        t._set(kw)
        return t

    @property
    def svd_prec(self):
        if self._svd_prec is None:
            self._svd_prec = math.sqrt(sys.float_info.epsilon)
        return self._svd_prec

    @svd_prec.setter
    def svd_prec(self, v):
        self._svd_prec = v


class ConvergenceError(Exception): pass

class CodeWrapError(Exception): pass

class CodeWrapper(object):

    _module_counter = 0

    def __init__(self, d, code, tempdir=None, verbose=False, flags=[]):
        self.code = code
        if tempdir is not None:
            tempdir = os.path.abspath(tempdir)
        self.filepath = tempdir
        self.flags = flags
        self.quiet = not verbose
        packages = ["eigen3"]
        if d["method"] in ("hybr", "lm"):
            packages.append('cminpack')
        dct = pkgconfig(*packages)
        libs = dct.get("libraries",[])
        libs.append("m")
        if d["resample"]:
            libs.append("zita-resampler")
        self.script_dict = dict(
            includes = " ".join("-I%s" % v for v in dct.get("include_dirs",[])),
            libraries = " ".join("-l%s" % v for v in set(libs)),
            #defines = "-DCHECK_BOUNDS",
            defines = "",
            debug = False,
            sourcename = "%s_%d.cpp" % (d["id"], CodeWrapper._module_counter),
            soname = "%s_%d.so" % (d["id"], CodeWrapper._module_counter),
            soname_ = "%s_%d_.so" % (d["id"], CodeWrapper._module_counter),
            optimize = False,#(d["method"] == "table"),
            )
        self.build_script = "./build_script_%d" % CodeWrapper._module_counter

    def _process_files(self, routine):
        command = self.command
        command.extend(self.flags)
        null = open(os.devnull, 'w')
        try:
            if self.quiet:
                retcode = subprocess.call(command, stdout=null, stderr=subprocess.STDOUT)
            else:
                retcode = subprocess.call(command)
        except OSError:
            retcode = 1
        if retcode:
            raise CodeWrapError(
                    "Error while executing command: %s" % " ".join(command))

    @property
    def command(self):
        command = [self.build_script]
        return command

    def _prepare_files(self):
        with open(self.build_script, 'w') as f:
            print >> f, dk_templates.build_script_template.render(self.script_dict)
            os.fchmod(f.fileno(), 0o777)
        

    def _generate_code(self):
        for fname, content in self.code.items():
            if fname == "c_source":
                fname = self.script_dict['sourcename']
            with open(fname, 'w') as f:
                f.write(content)

    def wrap_code(self, load_module=True):
        workdir = self.filepath or tempfile.mkdtemp("_dk_compile")
        if not os.access(workdir, os.F_OK):
            os.mkdir(workdir)
        oldwork = os.getcwd()
        os.chdir(workdir)
        try:
            self._generate_code()
            self._prepare_files()
            self._process_files(None)
            path = os.path.join(workdir, self.script_dict["soname"])
            if load_module:
                return path, SimulateC(path)
            else:
                return path
        finally:
            CodeWrapper._module_counter +=1
            os.chdir(oldwork)
            if not self.filepath:
                shutil.rmtree(workdir)


class LinearFilter(object):

    def __init__(self, p, jacobi):
        self.in_mat = p.N["I"]
        self.out_mat = p.N["O"]
        if self.in_mat.shape[0] != 1 or self.out_mat.shape[0] != 1:
            raise ValueError("linear filter generation only implemented for circuits with one input and one output channel")
        pot_func = p.get_pot_funcs()
        Rv = sp.diag(*[1/(f * v) for (a, f), v in zip(pot_func, p.Pv)])
        Nv = p.N["P"]
        S = p.S + Nv.T * Rv * Nv
        if jacobi is not None:
            S -= p.N["Nr"].T * jacobi * p.N["Nl"]
        self.S = S
        ss = set()
        self.syms = []
        for a, f in pot_func:
            if a not in ss:
                self.syms.append(a)
                ss.add(a)
        self.subst_var_default = p.get_variable_defaults()

    def get_s_coeffs(self):
        s = sp.symbols("s")
        expr = self.solve(self.S, self.in_mat, self.out_mat)
        r = [sp.poly(e, s) for e in sp.fraction(expr)]
        tc = r[1].TC()
        for v in tc.atoms(sp.Symbol):
            tc = tc.subs(v, 1)
        b_coeffs, b_terms = self.collect_s_coeffs(sp.poly(r[0], s), 'b', tc)
        a_coeffs, a_terms = self.collect_s_coeffs(sp.poly(r[1], s), 'a', tc)
        if a_coeffs[0] == -1:
            a_coeffs *= -1
            b_coeffs *= -1
        return b_coeffs, a_coeffs, b_terms / a_terms

    def collect_s_coeffs(self, expr, prefix, tc):
        s = sp.symbols("s")
        monoms = expr.monoms()
        max_degree = monoms[0][0]
        filter_coeffs = np.zeros(max_degree+1, dtype=object)
        ll = 0
        for e, i in zip(expr.coeffs(), monoms):
            i = i[0]
            if self.syms:
                # factorize according to variable symbols
                x = sp.poly(e, self.syms)
                ss = 0
                for co, o in zip(x.coeffs(), x.monoms()):
                    ss += reduce(operator.mul, [pow(y, p) for y, p in zip(self.syms, o)], 1) * (co/tc).simplify()
            else:
                ss = e/tc
            filter_coeffs[i] = ss
            sym = "%s%d" % (prefix, i)
            ll += sp.symbols(sym) * pow(s, i)
        return filter_coeffs, ll

    def transform_bilinear(self, expr):
        s = sp.symbols("s")
        fs = sp.symbols("fs")
        c = sp.symbols("c")
        z = sp.symbols("z")
        b = c*(z-1)/(z+1)
        r = [sp.poly(e, z) for e in sp.fraction(expr.subs(s, b).ratsimp())]
        return self.collect_z_symbolic(r[0]), self.collect_z_symbolic(r[1]), 2 * fs

    def collect_z_symbolic(self, expr):
        c = sp.symbols("c")
        monoms = expr.monoms()
        max_degree = monoms[0][0]
        filter_coeffs = np.zeros(max_degree+1, dtype=object)
        for e, i in zip(expr.coeffs(), monoms):
            idx = max_degree - i[0]
            filter_coeffs[idx] = e
        return filter_coeffs

    def convert_variable_dict(self, subst_var):
        d = self.subst_var_default.copy()
        df = set(subst_var.keys()) - set([str(v) for v in d.keys()])
        if df:
            raise ValueError("unknown variable(s): %s" % ", ".join(df))
        d.update(dict([(sp.symbols(k), v) for k, v in subst_var.items()]))
        return d

    def get_z_coeffs(self, samplerate=None, subst_var=None, as_expr=True):
        if samplerate is None:
            c = 2 * sp.symbols("fs")
        else:
            c = 2 * samplerate
        z = sp.symbols("z")
        b = c*(z-1)/(z+1)
        expr = self.S.subs(sp.symbols("s"), b)
        expr = self.solve(expr, self.in_mat, self.out_mat)
        if subst_var is not None:
            expr = expr.subs(subst_var)
            syms = None
        else:
            syms = self.syms
        # divide coeffs by magnitude of trailing denominator coefficient == a1 coefficient of filter
        lc = sp.poly(sp.fraction(expr)[1], z).LC()
        lc = lc.subs(self.subst_var_default)
        if samplerate is None:
            lc = lc.subs(sp.symbols("fs"), 48000.)
        r = [sp.poly((e/lc).expand(), z) for e in sp.fraction(expr)]
        return (self.collect_z_coeffs(r[0], 'b', syms, as_expr),
                self.collect_z_coeffs(r[1], 'a', syms, as_expr))

    def collect_z_coeffs(self, expr, prefix, syms, as_expr=True):
        monoms = expr.monoms()
        if as_expr:
            max_degree = monoms[0][0]
            filter_coeffs = np.zeros(max_degree+1, dtype=object)
        else:
            l = []
        for e, i in zip(expr.coeffs(), monoms):
            i = i[0]
            if syms:
                # factorize according to variable symbols
                x = sp.poly(e.expand(), syms)
                if not as_expr:
                    l.append(zip(x.monoms(), x.coeffs()))
                else:
                    ss = 0
                    for co, o in zip(x.coeffs(), x.monoms()):
                        ss += reduce(operator.mul, [pow(y, p) for y, p in zip(syms, o)], 1) * co.evalf()
            else:
                if not as_expr:
                    l.append(((0,), e))
                else:
                    ss = e.evalf()
            if as_expr:
                idx = max_degree-i
                filter_coeffs[idx] = sp.horner(ss, wrt=(syms or [])+[sp.symbols("fs")])
        if as_expr:
            return filter_coeffs
        else:
            return l, syms

    def coeffs_as_faust_code(self, prefix, coeffs):
        l = []
        for i, c in enumerate(coeffs):
            c = re.sub(r'([a-zA-Z0-9]+)\*\*(\d+)', r'pow(\1,\2)', str(c))
            l.append('%s%d = %s;' % (prefix, i, c))
        return l

    def print_coeffs(self, prefix, coeffs, f=sys.stdout):
        print >>f, "\n\n".join(self.coeffs_as_faust_code(prefix, coeffs))

    def spectrum(self, b, a, start_freq=20, stop_freq=10000, fs=48000, nbins=8*1024):
        b = [float(x) for x in b]
        a = [float(x) for x in a]
        w, h = sig.freqz(b, a, nbins)
        cut = slice(int(round(2 * nbins * start_freq / fs)), int(round(2 * nbins * stop_freq / fs)) + 1)
        w = (w[cut] * (fs/(2*np.pi)))
        h = 20*np.log10(abs(h[cut]))
        return w, h

    def solve(self, S, in_mat, out_mat):
        v = sp.Matrix(sp.symbols("v:%d" % self.S.shape[0]))
        try:
            p = subprocess.Popen(("maxima","-b","/dev/fd/0","--very-quiet"),
                                 stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
        except OSError as e:
            raise RuntimeError(
                "can't start maxima -- please check maxima installation [%s]" % e)
        out, expr = p.communicate(
            "stringout(\"/dev/fd/2\",facsum(linsolve([%s], [%s])[%d], s));\n" % (
                ", ".join(["%s = %s" % e for e in zip(S * v, in_mat)]),
                ", ".join([str(sym) for sym in v]),
                np.array(out_mat).nonzero()[1]+1,
                ))
        def maxima_error(s):
            print out
            print "----"
            print "Truncated output: " + expr[:100] + "..."
            raise ValueError(s)
        syms = set()
        for i in S:
            syms |= i.atoms(sp.Symbol)
        e = expr.split("=",1)
        if len(e) != 2:
            maxima_error("unexpected output from maxima")
        e = e[1].rstrip(";\n").replace("^","**")
        try:
            return eval(e, dict([(str(sym),sym) for sym in syms]))
        except Exception as ex:
            maxima_error("can't eval maxima output [%s]" % ex)

def get_state_transform_trace(A, B, C):
    "return the trace of the gram matrices (sensitivity measure)"
    Wc = ml.matrix(sla.solve_discrete_lyapunov(A.A, (B*B.T).A))
    Wo = ml.matrix(sla.solve_discrete_lyapunov(A.T.A, (C.T*C).A))
    #R = ml.matrix(sla.cholesky(Wo, lower=False))
    #X = R * Wc * R.T
    #U, s, V = sla.svd(X)
    #U = ml.matrix(U)
    #V = ml.matrix(V)
    #lli = np.sqrt(np.sqrt(s))
    #U = R.I * V.T * ml.diag(lli)
    #Ui = ml.diag(1/lli) * V * R
    #return U, Ui
    return np.trace(Wc), np.trace(Wo)

def get_state_transform(A, B, C, tol=0):
    "calculate balanced reduced state space model realization"
    # balance matrices
    A, B, C, maxred, scale = slicot.tb01id('A', A, B, C)
    # transform to schur form
    A, B, C, U, WR, WI = slicot.tb01wd(A, B, C)
    # reduce and balance
    T, Ti, A, B, C, hsv = slicot.ab09ax('D', 'B', A, B, C, tol=tol)
    # return transformation matrix (+ inverse) and hankel singular values
    return (np.matrix(np.diag(scale).dot(U).dot(T)),
            np.matrix(Ti.dot(U.T).dot(np.diag(1/scale))),
            hsv)


class NonlinEquations(object):

    def __init__(self, eq, v_slice):
        self.eq = eq
        self.v_slice = v_slice
        self.p_slice = v_slice
        self.i_slice = v_slice
        self.nn = self.v_slice.stop - self.v_slice.start
        self.nni = self.nn
        self.nno = self.nn
        self.U = self.Mi = ml.matrix(np.identity(self.nn))
        self.Hc = ml.zeros((self.nn, 1))
        self.pins = eq.nlin_elements[v_slice]
        self.name = "%s:%d" % (Parser.format_element(np.sort(self.pins, axis=0)[0]), self.nn)
        self.subblocks = []

    @staticmethod
    def create(eq, K, CZ, v_slice, opts):
        "return a Permution and an instance of NonlinEquations or derived"
        Pn = np.arange(len(CZ))
        # permute nonlinear part to make left upper submatrix of K blockdiagonal
        p, blocklist = NonlinEquations.get_block_indices(K[v_slice][:,v_slice], CZ[v_slice])
        if len(blocklist) > 1:
            Pn[v_slice] = p
            K = K[Pn][:,Pn]
            CZ = CZ[Pn]
        if opts.partition and len(blocklist) > 1:
            nlin = PartitionedNonlinEquations(eq, v_slice)
            Pn = Pn[nlin.create(eq, K, CZ, blocklist, opts)]
            return Pn, nlin
        # permute the system if there is more than one strongly connected component
        p, blocklist = NonlinEquations.find_scc(K[v_slice][:,v_slice], eq.get_parser())
        if opts.chain and len(blocklist) > 1:
            Pn[v_slice] = p
            K = K[Pn][:,Pn]
            CZ = CZ[Pn]
            nlin = ChainedNonlinEquations(eq, v_slice)
            Pn = Pn[nlin.create(eq, K, CZ, blocklist, opts)]
            return Pn, nlin
        return np.arange(len(CZ)), NonlinEquations(eq, v_slice)

    @staticmethod
    def rebase_nonlinear_functions(f, Pn):
        Pni = np.argsort(Pn) # inverse permutation
        a = np.zeros(len(f), dtype=object)
        for j, (expr, vl, base) in enumerate(f[Pn]):
            a[j] = (expr, vl, Pni[base])
        return a

    @staticmethod
    def get_unique_rows(a):
        """returns unique rows of a 2-dim array
        """
        if not a.size:
            return a
        order = np.lexsort(a.T)
        a = a[order]
        diff = np.diff(a, axis=0)
        ui = np.ones(len(a), dtype=bool)
        ui[1:] = (diff != 0).any(axis=1) 
        return a[ui]

    @staticmethod
    def get_blockdiag_permutation(B):
        """returns the permuted index list to transform B to block diagonal form

        B: quadratic (sparse) matrix
        returns: index permutation (p), block indexlist (bl)

        To get the block diagonal matrix A:
        A = B[p][:,p]

        Permutation matrix:
        Q = matrix(eye(len(p)))[p]
        A = Q * B * Q.T

        To permute a list of row or column labels with the matrix:
        labels_for_A = [labels_for_B[i] for i in p]

        To get the ith block matrix:
        r = slice(bl[i], bl[i+1])
        A[r,r]
        """
        B = ml.matrix(B, bool)
        G = B.T * B
        while True:
            G1 = G * G
            if (G1 == G).all():
                break
            G = G1
        n = B.shape[0]
        p = []
        bl = [0]
        for row in NonlinEquations.get_unique_rows(G.A):
            i = np.nonzero(row)[0]
            if len(i):
                p.extend(i)
                bl.append(len(p))
        if len(p) != B.shape[0]:
            p.extend(set(range(B.shape[0])) - set(p))
        return p, bl

    @staticmethod
    def get_block_indices(K, CZ):
        "returns a permutation and a list of component slices"
        nz = len(CZ)
        n = np.count_nonzero(CZ)
        if n != nz:
            pc = np.argsort(CZ == 0)
            K = K[pc][:,pc][:n,:n]
        p, bl = NonlinEquations.get_blockdiag_permutation(K)
        if n != nz:
            p = np.array(pc)[p+range(n, nz)]
        return p, [slice(i,j) for i, j in zip(bl[:-1], bl[1:])]

    @staticmethod
    def decompose(a):
        U, s, V = la.svd(a, full_matrices=False)
        o = np.sqrt(np.sum(s*s) / np.count_nonzero(s))
        sr = np.where(s, s, o)
        V1 = ml.diag(sr) * V
        V1i = (ml.diag(1/sr) * V).T
        U1 = U * np.diag(np.where(s, 1, 0))
        if (U1 <= 0).all():
            U1 = -U1
            V1 = -V1
            V1i = -V1i
        return U1, V1, V1i

    @staticmethod
    def find_scc(K, parser):
        from scipy.sparse import csr_matrix
        from scipy.sparse.csgraph import connected_components
        n, label = connected_components(csr_matrix(K), connection="strong")
        V = np.zeros(n)
        L = [[] for i in range(n)]
        for i, l in enumerate(L):
            col = K[:,label==i]
            for j in range(n):
                if i != j and col[label==j].any():
                    l.append(j)
                    V[j] += 1
        p = []
        l = []
        N = np.arange(len(K))
        while True:
            idx = np.argwhere(V == 0)
            if not idx.size:
                break
            V[idx] -= 1
            for i in idx:
                pi = N[label==i]
                j = len(p)
                l.append(slice(j, j+len(pi)))
                p.extend(pi)
                V[L[int(i)]] -= 1
        return p, l

    def make_input_trans(self, matrix_list, opts):
        if self.eq.np or opts.svd_prec < 0:
            return matrix_list
        # find number of linear independent inputs to the nonlinear function
        M = np.concatenate(matrix_list, axis=1)
        if not M.size:
            return matrix_list
        U, SV, V = la.svd(M, full_matrices=False)
        if SV[0] != 0:
            SV = np.where(SV / SV[0] > opts.svd_prec, SV, 0)
        nni = np.count_nonzero(SV)
        if nni == self.nn:
            return matrix_list
        U = U[:,:nni]
        UH = U.H
        self.U = U
        self.nni = nni
        self.p_slice = slice(self.p_slice.start, self.p_slice.start+nni)
        return [UH * m for m in matrix_list]

    def make_output_trans(self, matrix_list, opts):
        if self.nn == 0 or self.eq.np != 0 or opts.svd_prec < 0:
            return matrix_list
        # find number of linear independent outputs of the nonlinear function
        M = np.concatenate(matrix_list, axis=0)
        U, SV, V = la.svd(M, full_matrices=False)
        if SV[0] != 0:
            SV = np.where(SV / SV[0] > opts.svd_prec, SV, 0)
        n = np.count_nonzero(SV)
        if n < self.nn and n < self.eq.nx + self.eq.no:
            self.nno = n
            self.Mi = (np.diag(SV) * V)[:n]
            s = np.cumsum([0]+[m.shape[0] for m in matrix_list])
            matrix_list = [U[slice(*sl),:n] for sl in zip(s[:-1],s[1:])]
        elif self.eq.nx + self.eq.no < self.nn:
            self.nno = self.eq.nx + self.eq.no
            self.Mi = M
            M = np.eye(self.nno, self.nno)
            s = np.cumsum([0]+[m.shape[0] for m in matrix_list])
            matrix_list = [M[slice(*sl)] for sl in zip(s[:-1],s[1:])]
        self.i_slice = slice(self.i_slice.start, self.i_slice.start+self.nno)
        return matrix_list

    def transform_p0(self, p0, K, v0):
        return p0


class ChainedNonlinEquations(NonlinEquations):

    def create(self, eq, K, CZ, blocklist, opts):
        self.subblocks = []
        Pn = np.arange(len(K))
        for sl in blocklist:
            p, nlin = NonlinEquations.create(eq, K, CZ, sl, opts.copy(partition=False))
            self.subblocks.append(nlin)
            Pn = Pn[p]
        return Pn

    def make_input_trans(self, matrix_list, opts):
        if opts.svd_prec < 0:
            return matrix_list
        i = self.subblocks[0].p_slice.start
        j = self.subblocks[-1].p_slice.stop
        ll = [[] for m in matrix_list]
        if i:
            for l, m in zip(ll, matrix_list):
                l.append(m[0:i])
        for bl in self.subblocks:
            r = bl.make_input_trans([m[bl.p_slice] for m in matrix_list], opts)
            for l, m in zip(ll, r):
                l.append(m)
        for l, m in zip(ll, matrix_list):
            if j < m.shape[0]:
                l.append(m[j:])
        return [np.concatenate(l, axis=0) for l in ll]

    def make_output_trans(self, matrix_list, opts):
        if opts.svd_prec < 0:
            return matrix_list
        F, C = matrix_list ##FIXME
        i = self.subblocks[0].i_slice.start
        j = self.subblocks[-1].i_slice.stop
        l_F = []
        l_C = []
        if i:
            l_F.append(F[:,0:i])
            l_C.append(C[:,0:i])
        for bl in self.subblocks:
            Fs, Cs = bl.make_output_trans((F[:,bl.i_slice], C[:,bl.i_slice]), opts)
            l_F.append(Fs)
            l_C.append(Cs)
        if j < F.shape[0]:
            l_F.append(F[:,j:])
            l_C.append(C[:,j:])
        return np.concatenate(l_F, axis=1), np.concatenate(l_C, axis=1)


class PartitionedNonlinEquations(NonlinEquations):

    def create(self, eq, K, CZ, blocklist, opts):
        self.cc_slice = slice(blocklist[-1].stop, self.v_slice.stop)
        self.subblocks = []
        Pn = np.arange(len(K))
        for sl in blocklist:
            p, nlin = NonlinEquations.create(eq, K, CZ, sl, opts.copy(chain=False))
            self.subblocks.append(nlin)
            Pn = Pn[p]
        return Pn

    def transform_p0(self, p0, K, v0):
        endv = self.cc_slice.start
        endp = self.p_slice.stop - (self.cc_slice.stop - self.cc_slice.start)
        p = p0.copy()
        p[:endp] = (p0 + self.Hc)[:endp] + self.Ku * np.matrix(v0[endv:]).T
        return p

    def make_input_trans(self, matrix_list, opts):
        if opts.svd_prec < 0:
            Ku = [self.eq.K[bl.p_slice,self.cc_slice] for bl in self.subblocks]
            self.Ku = np.concatenate(Ku, axis=0)
            return matrix_list
        l = []
        j = 0
        Ku = []
        for bl in self.subblocks:
            mlist = [m[bl.p_slice] for m in matrix_list]
            mlist.append(self.eq.K[bl.p_slice,self.cc_slice])
            bl.Hc = self.Hc[bl.p_slice]
            mlist = bl.make_input_trans(mlist, opts)
            bl.p_slice = slice(j, bl.p_slice.stop - (bl.p_slice.start - j))
            j = bl.p_slice.stop
            l.append(mlist[:-1])
            Ku.append(mlist[-1])
        self.Ku = np.concatenate(Ku, axis=0)
        Hc = self.Hc[self.cc_slice]
        l.append([m[self.cc_slice] for m in matrix_list])
        n_old = self.p_slice.stop - self.p_slice.start
        cc = self.cc_slice.stop - self.cc_slice.start
        self.p_slice = slice(self.p_slice.start, j+cc)
        self.nni = self.p_slice.stop - self.p_slice.start
        self.Hc = ml.zeros((self.nni, 1))
        self.Hc[j:] = Hc
        self.U = ml.eye(n_old, self.nni, self.nni-n_old)
        j = 0
        for bl in self.subblocks:
            self.U[j:j+bl.U.shape[0],bl.p_slice] = bl.U
            j += bl.U.shape[0]
        return [np.concatenate(m, axis=0) for m in zip(*l)]

    def make_output_trans(self, matrix_list, opts):
        if opts.svd_prec < 0:
            Kl = [self.eq.K[self.cc_slice, bl.i_slice] for bl in self.subblocks]
            Kl.append(self.eq.K[self.cc_slice, self.cc_slice])
            self.Kl = np.concatenate(Kl, axis=1)
            return matrix_list
        l = []
        j = 0
        Kl = []
        for bl in self.subblocks:
            mlist = [m[:,bl.i_slice] for m in matrix_list]
            mlist.append(self.eq.K[self.cc_slice, bl.i_slice])
            mlist = bl.make_output_trans(mlist, opts)
            bl.i_slice = slice(j, bl.i_slice.stop - (bl.i_slice.start - j))
            j = bl.i_slice.stop
            l.append(mlist[:-1])
            Kl.append(mlist[-1])
        Kl.append(self.eq.K[self.cc_slice, self.cc_slice])
        self.Kl = np.concatenate(Kl, axis=1)
        l.append([m[:,self.cc_slice] for m in matrix_list])
        n_old = self.i_slice.stop - self.i_slice.start
        cc = self.cc_slice.stop - self.cc_slice.start
        self.i_slice = slice(self.i_slice.start, j+cc)
        self.nno = self.i_slice.stop - self.i_slice.start
        self.Mi = ml.eye(self.nno, n_old, n_old-self.nno)
        j = 0
        for bl in self.subblocks:
            self.Mi[bl.i_slice, j:j+bl.Mi.shape[1]] = bl.Mi
            j += bl.Mi.shape[1]
        return [np.concatenate(m, axis=1) for m in zip(*l)]

    def decompose_blocks(self, F, C):
        Kni = ml.identity(self.nn)
        end = self.subblocks[-1].v_slice.stop
        #self.Kl = self.eq.K[end:, :].copy()
        #return F, C
        for bl in self.subblocks:
            sl = bl.v_slice
            U, V, Vi = self.decompose(self.eq.K[end:, sl])
            m = ml.zeros_like(self.Kl[:,sl])
            m[:,:U.shape[1]] = U
            self.Kl[:,sl] = m
            #self.Mi[sl,sl] = V
            bl.Mi = V
            m = ml.zeros_like(Kni[sl,sl])
            m[:,:Vi.shape[1]] = Vi
            Kni[sl,sl] = m
        return F * Kni, C * Kni


class EquationSystem(object):

    def __init__(self, parser, jacobi_par=None, opts=None):
        self.parser = parser
        self.jacobi_par = jacobi_par
        if opts is None:
            opts = TransformOptions()

        self.nx, self.nn, self.ni, self.no, self.np = [len(parser.element_name[j]) for j in 'X', 'N', 'I', 'O', 'P']
        self.nlin_elements = np.array(parser.element_name["N"])
        self.Nxl, self.Nxr, self.Nnl, self.Nnr, self.No, self.Nv, self.I = [
            parser.N[j] for j in "Xl", "Xr", "Nl", "Nr", "O", "P", "I"]
        self.CV = parser.ConstVoltages
        m = parser.mm
        self.f = parser.get_nonlin_funcs()
        self.CZ = parser.CZ
        if self.jacobi_par is not None:
            J, Jc, select, unselect = self.jacobi_par
            self.S = parser.S - self.Nnr[unselect].T * J[unselect][:,unselect] * self.Nnl[unselect]
            self.CV = self.CV + (self.Nnr[unselect].T * Jc[unselect]).T
            self.Nnr = self.Nnr[select]
            self.Nnl = self.Nnl[select]
            self.nn = len(select)
            self.CZ = self.CZ[select]
            self.f = NonlinEquations.rebase_nonlinear_functions(self.f, select)
            parser.f = self.f
            parser.element_name["N"] = np.array(parser.element_name["N"])[select]
        else:
            self.S = parser.S
        self.Si = self.S.I

        if self.nn:
            K = (self.Nnl * self.Si * self.Nnr.T != 0)
            for j, (expr, vl, base) in enumerate(self.f):
                for i in base:
                    K[j, i] = True
            v_slice = slice(0, len(K))
            Pn, self.nonlin = NonlinEquations.create(self, K, self.CZ, v_slice, opts)
            self.apply_permutation(Pn)
        else:
            self.nonlin = None

        Z = ml.diag(parser.Z)
        self.Tx = m * Z * self.Nxl * self.Si
        self.A = self.Tx * self.Nxr.T - (Z if parser.TR else ml.diag((parser.Z - 1) / 2.0))
        self.B = self.Tx * self.I.T
        self.Bc = self.Tx * self.CV.T
        self.C = self.Tx * self.Nnr.T

        self.To = self.No * self.Si
        self.D = self.To * self.Nxr.T
        self.E = self.To * self.I.T
        self.Ec = self.To * self.CV.T
        self.F = self.To * self.Nnr.T

        self.Tn = self.Nnl * self.Si
        self.G = self.Tn * self.Nxr.T
        self.H = self.Tn * self.I.T
        self.Hc = self.Tn * self.CV.T
        self.K = self.Tn * self.Nnr.T

        if self.nn:
            self.nonlin.Hc = self.Hc
            self.G, self.H = self.nonlin.make_input_trans((self.G, self.H), opts)
            self.F, self.C = self.nonlin.make_output_trans((self.F, self.C), opts)

        if self.np:
            # Woodbury Identity: \left(A+UCV \right)^{-1} = A^{-1} - A^{-1}U \left(C^{-1}+VA^{-1}U \right)^{-1} VA^{-1}, 
            self.Twl = self.Si * self.Nv.T
            self.Q = self.Nv * self.Twl
            self.Uxl = m * Z * self.Nxl * self.Twl
            self.Uo = self.No * self.Twl
            self.Unl = self.Nnl * self.Twl
            self.Twr = self.Si.T * self.Nv.T
            self.Uxr = self.Nxr * self.Twr
            self.Uu = self.I * self.Twr
            self.Ucv = self.CV * self.Twr
            self.Unr = self.Nnr * self.Twr

        self.mp_cols = self.nx + self.ni

    def apply_permutation(self, Pn):
        self.nlin_elements = self.nlin_elements[Pn]
        self.CZ = self.CZ[Pn]
        self.f = NonlinEquations.rebase_nonlinear_functions(self.f, Pn)
        self.Nnl = self.Nnl[Pn]
        self.Nnr = self.Nnr[Pn]

    def transform_state(self, Ts, Tsi):
        self.A = Tsi * self.A * Ts
        self.B = Tsi * self.B
        self.Bc = Tsi * self.Bc
        self.C = Tsi * self.C
        self.D = self.D * Ts
        self.G = self.G * Ts
        self.nx = self.A.shape[0]
        self.mp_cols = self.nx + self.ni
        if self.np:
            self.Uxl = Tsi * self.Uxl
            self.Uxr = Ts.T * self.Uxr

    def get_parser(self):
        return self.parser

    def get_Mo(self):
        return np.concatenate((self.D, self.E, self.F), axis=1)

    def get_Mx(self):
        return np.concatenate((self.A, self.B, self.C), axis=1)

    def get_Mp(self):
        return np.concatenate((self.G, self.H), axis=1)

    def get_UR(self):
        assert hasattr(self, "Q")
        return np.concatenate((self.Uxr.T, self.Uu.T, self.Unr.T), axis=1)

    def get_mx_cols(self):
        return self.A.shape[1] + self.B.shape[1] + self.C.shape[1]

    def get_mp_cols(self):
        return self.G.shape[1] + self.H.shape[1]

    def get_npl(self):
        return len(self.parser.pot_list)

    def get_status(self):
        return "nx=%d, nni=%d, nn=%d, nno=%d, ni=%d, no=%d, np=%d" % (self.nx, self.nni, self.nn, self.nno, self.ni, self.no, self.np)


class Simulate(object):

    def __init__(self, eq, solver):
        self.eq = eq
        self.parser = eq.get_parser()
        if solver is None:
            self.solver_dict = dict()
        elif isinstance(solver, basestring):
            self.solver_dict = dict(method = solver)
        else:
            self.solver_dict = dict(solver)
        self.solver_method = self.solver_dict.get("method", 'hybr')
        self.max_homotopy_iter = self.solver_dict.get("max_homotopy_iter", 1000)
        try:
            del self.solver_dict["max_homotopy_iter"]
        except KeyError:
            pass
        try:
            del self.solver_dict["method"]
        except KeyError:
            pass

    def get_solver(self):
        d = self.solver_dict.copy()
        d["method"] = self.solver_method
        d["max_homotopy_iter"] = self.max_homotopy_iter
        return d

    def get_eq(self):
        return self.eq

    def get_parser(self):
        return self.parser


class SimulatePy(Simulate):

    def __init__(self, eq, solver=None, dc_method="A"):
        Simulate.__init__(self, eq, solver)
        self.dc_method = dc_method
        parser = eq.get_parser()
        self.pot_func = parser.get_pot_funcs()
        self.Pv = parser.Pv
        self.pot = parser.pot
        self.out_labels = parser.out_labels()
        self.v0 = parser.V.get("v0", np.zeros(eq.nn))
        self.x = np.zeros(eq.nx)
        self.o0 = np.zeros(eq.no)
        self.op = parser.op
        self.compile_py_func()
        self.calc_dc(parser.op, method=dc_method)

    def set_variable(self, var, val):
        self.pot[var] = val

    def calc_Rv(self):
        n = self.eq.np
        Rv = ml.matrix(np.zeros((n, n)))
        for i, ((a, f), p) in enumerate(zip(self.pot_func, self.Pv)):
            k = str(a)
            try:
                v = self.pot[k]
            except KeyError:
                v = self.pot[k] = 0.5
            Rv[i, i] = float(f.subs({a: v})) * p
        return Rv

    def calc_matrixes(self):
        eq = self.eq
        if eq.np:
            Qi = (eq.Q + self.calc_Rv()).I
            Tx = eq.Uxl * Qi
            To = eq.Uo * Qi
            Tn = eq.Unl * Qi
            return (eq.A - Tx * eq.Uxr.T,
                    eq.B - Tx * eq.Uu.T,
                    eq.Bc - Tx * eq.Ucv.T,
                    eq.C - Tx * eq.Unr.T,
                    eq.D - To * eq.Uxr.T,
                    eq.E - To * eq.Uu.T,
                    eq.Ec - To * eq.Ucv.T,
                    eq.F - To * eq.Unr.T,
                    eq.G - Tn * eq.Uxr.T,
                    eq.H - Tn * eq.Uu.T,
                    eq.Hc - Tn * eq.Ucv.T,
                    eq.K - Tn * eq.Unr.T,
                    )
        else:
            return (eq.A, eq.B, eq.Bc, eq.C,
                    eq.D, eq.E, eq.Ec, eq.F,
                    eq.G, eq.H, eq.Hc, eq.K,
                    )

    def calc_linear_sys_matrices(self, v0=None):
        eq = self.eq
        if eq.nn:
            nonlin = eq.nonlin
            J = self.jacobi(v0)
            X = nonlin.Mi * (np.diag(eq.CZ) - J * eq.K).I * J * nonlin.U
            # equivalent formula for X:
            # X = nonlin.Mi * J * (np.diag(eq.CZ) - eq.K * J).I * nonlin.U
            A = eq.A + eq.C * X * eq.G
            B = eq.B + eq.C * X * eq.H
            D = eq.D + eq.F * X * eq.G
            return A, B, D
        else:
            return eq.A, eq.B, eq.D

    def balance_realization(self, tol=None):
        if self.eq.np:
            return
        A, B, D = self.calc_linear_sys_matrices()
        T, Ti, hsv = get_state_transform(A, B, D, tol=tol)
        logger = logging.getLogger("balance")
        if T.shape[0] != T.shape[1]:
            logger.info("reduced system size %d -> %d" % T.shape)
        logger.debug("HSV = %s" % hsv)
        self.eq.transform_state(T, Ti)
        self.calc_dc(self.eq.parser.op, method=self.dc_method)

    def solve(self, func, v0, args=(), method='hybr', options=None):
        if method == "lm" and options and "maxfev" in options:
            options["maxiter"] = options["maxfev"]
            del options["maxfev"]
        try:
            with warnings.catch_warnings():
                warnings.filterwarnings(action="error")
                res = opt.root(func, v0, args=args, method=method, options=options)
        except RuntimeWarning as e:
            raise ConvergenceError(e)
        else:
            if not res.success:
                raise ConvergenceError(res.message)
        return res

    def solve_using_homotopy(self, func, v0, method='hybr', options=None):
        # use homotopy
        points = [0, 1]
        max_iter = 100
        for tries in range(max_iter):
            try:
                res = self.solve(func, v0, args=(points[1],), method=method, options=options)
            except ConvergenceError as e:
                msg = e
                points.insert(1, (points[0]+points[1])/2)
                continue
            if len(points) == 2:
                return res
            v0 = res.x
            points = points[1:]
        raise ConvergenceError("more than %d iterations (list msg: %s)" % (max_iter, msg))

    def calc_dc(self, u, method="A"):
        u = ml.matrix(u, dtype=np.float64).T
        A, B, Bc, C, D, E, Ec, F, G, H, Hc, K = self.calc_matrixes()
        if A.size == 0:
            if len(self.eq.f):
                p = self.eq.nonlin.U * H * u
                def func(v, fact):
                    return (p + Hc * fact + K * self.calc_i(v) - ml.matrix(self.eq.CZ * v).T).A1
                self.v0 = self.solve_using_homotopy(func, self.v0).x
        else:
            I = ml.eye(len(A))
            if method == "A":
                try:
                    Ai = (I - A).I
                except la.LinAlgError:
                    method = "N"
            if method == "N":
                if self.eq.nonlin is None:
                    Bu = B * u + Bc
                    def func(v, fact):
                        vv = ml.matrix(v).T
                        return ((A - I) * vv + Bu*fact).A1
                    res = self.solve_using_homotopy(func, self.x)
                    self.x = res.x
                else:
                    G1 = ml.append(self.eq.nonlin.U * G, A, axis=0)
                    H1 = ml.append(self.eq.nonlin.U * H, B, axis=0)
                    K1 = ml.append(K, C * self.eq.nonlin.Mi, axis=0)
                    Hc1 = ml.append(Hc, Bc, axis=0)
                    CZ1 = np.append(self.eq.CZ, np.ones(len(self.x)))
                    n = len(self.v0)
                    def func(v, fact):
                        return (G1 * ml.matrix(v[n:]).T + H1 * u + Hc1*fact + K1 * self.calc_i(v) - ml.matrix(CZ1 * v).T).A1
                    res = self.solve_using_homotopy(func, np.append(self.v0, self.x))
                    self.v0 = res.x[:n]
                    self.x = res.x[n:]
            else:
                self.x = matrix_add(Ai * B * u, Ai * Bc)
                if K.size != 0:
                    T = self.eq.nonlin.U * G * Ai
                    p = matrix_add(T * B * u, self.eq.nonlin.U * H * u)
                    Hc1 = matrix_add(T * Bc, Hc)
                    KK = T * C * self.eq.nonlin.Mi + K
                    def func(v, fact):
                        return (p + Hc1 * fact + KK * self.calc_i(v) - ml.matrix(self.eq.CZ * v).T).A1
                    self.v0 = self.solve_using_homotopy(func, self.v0).x
                    self.x += Ai * C * self.eq.nonlin.Mi * self.calc_i(self.v0)
                self.x = self.x.A1
        self.v00 = self.v0.copy()
        self.x0 = self.x.copy()
        self.p0 = matrix_add(G * np.matrix(self.x0).T, H * np.matrix(self.op).T)
        if self.eq.nonlin:
            self.p0 = self.eq.nonlin.transform_p0(self.p0, K, self.v0)
            self.o0 = matrix_add(D * np.matrix(self.x0).T, E * np.matrix(self.op).T, Ec, F * self.eq.nonlin.Mi * self.calc_i(self.v0)).A1
            self.last_p = self.eq.nonlin.U * self.p0 + Hc
        else:
            self.o0 = matrix_add(D * np.matrix(self.x0).T, E * np.matrix(self.op).T, Ec).A1

    def calc_i(self, v):
        i = ml.zeros(len(self.ff)).T
        for n, (f, base) in enumerate(self.ff):
            i[n] = f(*v[base])
        return i

    def solve_one(self, p, K, s, sm=None, sd=None):
        if sm is None:
            sm = self.solver_method
        if sd is None:
            sd = self.solver_dict
        i = ml.zeros((p.shape[0],1))
        vv = self.v0.copy()
        def func(v):
            for n, (f, base) in enumerate(self.ff[s]):
                ##FIXME
                vv[s] = v
                i[n] = f(*vv[base])
            return (p + K * i - ml.matrix(v).T).A1
        self.v0[s] = self.solve(func, self.v0[s], method=sm, options=sd).x
        return i

    def nonlin_py(self, p, K, Hc):
        p = self.eq.nonlin.U * p + Hc
        i = ml.zeros((self.eq.nn, 1))
        if isinstance(self.eq.nonlin, PartitionedNonlinEquations):
            rc = slice(self.eq.blocklist[-1].stop, None)
            #sm = ["hybr", "lm", "hybr"]
            #sd = [dict(), dict(diag=(1e3,1),factor=1e-1), dict()]
            #sm = ["lm", "lm", "lm"]
            #sd = [dict(diag=(1e3,1),factor=1e-1), dict(diag=(1e3,1),factor=1e-1), dict(diag=(1e3,1),factor=1e-1)]
            sm = [None,None,None]
            sd = [None,None,None]
            def func(icc, fact):
                #print "*", fact, icc
                p1 = self.last_p + (p - self.last_p) * fact
                for j, s in enumerate(self.eq.blocklist):
                    k = K[s]
                    try:
                        i[s] = self.eq.Kn[j] * self.solve_one(p1[s]+k[:,rc].dot(icc).T, k[:,s], s, sm[j], sd[j])
                    except ConvergenceError as e:
                        print "conv error in %d: %s" % (j, e)
                        raise
                        #raise SystemExit
                i[rc] = icc.reshape(len(icc), 1)
                return (p1[rc] + self.eq.Kl * i).A1
            self.v0[rc] = self.solve_using_homotopy(func, self.v0[rc], method=self.solver_method, options=self.solver_dict).x
        elif isinstance(self.eq.nonlin, ChainedNonlinEquations):
            for j, s in enumerate(self.eq.nonlin.blocklist):
                pp = p[s]
                if s.start:
                    pp += K[s][:,:s.start] * i[:s.start]
                i[s] = self.solve_one(pp, K[s][:,s], s)
            i = self.eq.nonlin.Mi * i
        else:
            def func(v, fact):
                i[:] = self.calc_i(v)
                p1 = self.last_p + (p - self.last_p) * fact
                return (p1 + K * i - ml.matrix(self.eq.CZ * v).T).A1
            self.v0 = self.solve_using_homotopy(func, self.v0, method=self.solver_method, options=self.solver_dict).x
            i = self.eq.nonlin.Mi * i
        self.last_p = p
        return i

    def compile_py_func(self):
        self.ff = np.zeros(len(self.eq.f), dtype=object)
        if not self.eq.nn:
            # model is linearized
            return
        for j, (expr, vl, base) in enumerate(self.eq.f):
            self.ff[j] = (sp.lambdify(vl, expr), base)

    def calc_di(self, v, j):
        i = np.zeros(len(self.eq.f))
        for n, (f, vl, base) in enumerate(self.eq.f):
            for k, var in zip(base, vl):
                if k == j:
                    s = dict([(sym,val) for sym, val in zip(vl, v[base])])
                    i[n] = f.diff(var).subs(s)
                    break
            else:
                i[n] = 0.
        return i

    def jacobi_numeric(self, v0=None):
        if v0 is None:
            v0 = self.v00
        J = np.zeros((len(self.eq.f), len(v0)))
        i0 = self.calc_i(v0).A1
        eps = 1e-4
        for j in range(len(v0)):
            v = v0.copy()
            v[j] += eps
            J[:, j] = (self.calc_i(v).A1 - i0) / eps
        return ml.matrix(J)

    def jacobi_symbolic(self, v0=None):
        if v0 is None:
            v0 = self.v00
        J = np.zeros((len(self.eq.f), len(v0)))
        for j in range(len(v0)):
            J[:, j] = self.calc_di(v0, j)
        return ml.matrix(J)

    jacobi = jacobi_symbolic
    #jacobi = jacobi_numeric

    def eval_py(self, v_in, ii=-1):
        self.x = ml.matrix(self.x).T ##FIXME
        v_in = ml.matrix(v_in)
        assert v_in.shape[1] == self.eq.ni
        y = np.empty((v_in.shape[0], self.eq.no))
        t1 = time.time()
        A, B, Bc, C, D, E, Ec, F, G, H, Hc, K = self.calc_matrixes()
        self.minmax = np.array(((float("inf"), float("-inf")),) * G.shape[0])
        for n, u in enumerate(v_in):
            u = ml.matrix(u).T
            if len(self.v0) == 0:
                i = ml.matrix(()).T
            else:
                try:
                    p = G * self.x + H * u
                    self.minmax[:,0] = np.min((self.minmax[:,0], np.ravel(p)), axis=0)
                    self.minmax[:,1] = np.max((self.minmax[:,1], np.ravel(p)), axis=0)
                    i = self.nonlin_py(p, K, Hc)
                except ConvergenceError as e:
                    print "##", n
                    raise
            y[n,:] = matrix_add(D * self.x, E * u, Ec, F * i).A1
            self.x = matrix_add(A * self.x, B * u, Bc, C * i)
        self.time_per_sample = (time.time() - t1)/(n+1)
        self.x = self.x.A1 ##FIXME
        return y

    def __call__(self, v_in, ii=-1):
        return self.eval_py(v_in, ii)


class CheckedDict(dict):
    def __setitem__(self, n, v):
        assert n not in self
        return dict.__setitem__(self, n, v)
    def overwrite(self, n, v):
        return dict.__setitem__(self, n, v)

class PluginDef(object):

    def __init__(self, id):
        self.id = id
        self.name = id
        self._description = None
        self.category = "External"
        self._shortname = None
        self.namespace = id
        self.lv2_plugin_type = None
        self.lv2_versioned_id = id
        self.lv2_minor_version = 0
        self.lv2_micro_version = 0

    @staticmethod
    def _lfmt(s):
        if s is None:
            return 0
        if not s:
            return '""'
        return 'N_("%s")' % s

    @property
    def description(self):
        return self._description or ""
    @description.setter
    def description(self, v):
        self._description = v

    @property
    def shortname(self):
        return self._shortname or ""
    @shortname.setter
    def shortname(self, v):
        self._shortname = v

    @property
    def s_id(self):
        return '"%s"' % self.id

    @property
    def l_name(self):
        return self._lfmt(self.name)

    @property
    def l_description(self):
        return self._lfmt(self._description)

    @property
    def l_category(self):
        return self._lfmt(self.category)

    @property
    def l_shortname(self):
        return self._lfmt(self._shortname)


class BuildCModule(Simulate):

    def __init__(self, name, sim, solver=None, c_tempdir=None, c_verbose=False,
                 c_real="double", extra_sources=None, linearize=False, pre_filter=None,
                 post_filter=None, generator=None):
        if solver is None:
            solver = sim.get_solver()
        Simulate.__init__(self, sim.get_eq(), solver)
        self.name = name
        self.c_tempdir = c_tempdir
        self.c_verbose = c_verbose
        self.c_real = c_real
        self.extra_sources = extra_sources
        self.pre_filter = pre_filter
        self.post_filter = post_filter
        if generator is None:
            generator = generate_code.CodeGenerator
        self.generator = generator
        parser = sim.get_parser()
        if linearize:
            l = [i for i, e in enumerate(parser.element_name["N"]) if "linearize" not in parser.V[e[0]]]
            li = [i for i, e in enumerate(parser.element_name["N"]) if "linearize" in parser.V[e[0]]]
            J = sim.jacobi()
            Jc = sim.calc_i(sim.v00)
            par = J, Jc, l, li
            self.eq = EquationSystem(parser, par)
            sim = SimulatePy(self.eq, solver)
        self.v0, self.x0, self.p0, self.o0 = sim.v0, sim.x0, sim.p0, sim.o0
        self.op = parser.op
        self.pot_func = parser.get_pot_funcs()
        self.pot = parser.pot
        self.Pv = parser.Pv
        self.pot_attr = parser.get_pot_attr()
        self.out_labels = parser.out_labels()
        self.fs = parser.fs
        self.max_homotopy_iter = 64000
        self.resample = False
        self.solver_params = None
        self.dev_interface = True
        self.build_script = None
        self.plugindef = PluginDef(name)

    def get_executor(self):
        return self.compile_c_func().wrap_code()

    def compile_c_func(self):
        eq = self.eq
        d = CheckedDict(name=self.name, comment=time.ctime())
        d["solver_maptype"] = "unsigned short" ##FIXME
        d["fs"] = self.fs
        d["resample"] = self.resample
        d["c_real"] = self.c_real
        for j in "nx", "nn", "ni", "no", "np", "mp_cols":
            d[j] = getattr(eq, j)
        for j in "nni", "nno":
            d[j] = getattr(eq.nonlin, j, 0)
        d["npl"] = self.eq.get_npl()
        d["v0_data"] = ",".join([str(j) for j in self.v0])
        d["x0_data"] = ",".join([str(j) for j in self.x0])
        d["p0_data"] = ",".join([str(j) for j in self.p0.A1])
        d["o0_data"] = ",".join([str(j) for j in self.o0])
        d["op_data"] = ",".join([str(j) for j in self.op])
        d["out_labels"] = ",".join(['"%s"' % j for j in self.out_labels])
        d["nlin_elements"] = [Parser.format_element(v) for v in self.eq.nlin_elements]
        d["method"] = method = "linear" if eq.nn == 0 else self.solver_method
        pot_list = self.eq.get_parser().pot_list
        d["pot_vars"] = ",".join(['"%s"' % v for v in pot_list])
        d["pot"] = ",".join([str(self.pot.get(v,0.5)) for v in pot_list])
        d["pre_filter"] = self.pre_filter or ""
        d["post_filter"] = self.post_filter or ""
        d["post_process"] = ""
        d['id'] = d["name"]
        d['plugindef'] = self.plugindef
        d['build_script'] = self.build_script
        d["dev_interface"] = self.dev_interface
        solver = self.solver_dict.copy()
        solver["method"] = method
        solver["max_homotopy_iter"] = self.max_homotopy_iter
        if method == "table":
            d["extra_sources"] = self.extra_sources
        else:
            d["extra_sources"] = ""
        cg = self.generator(
            self.eq, solver, self.solver_params, self.pot, pot_list,
            self.pot_func, self.pot_attr, self.Pv, self.extra_sources
            ).generate(d)
        return CodeWrapper(d, cg, self.c_tempdir, self.c_verbose)


class SimulateC(object):

    def __init__(self, soname):
        self.soname = soname
        self.load_from_shared_lib()
        self.v0, self.x, self.p0, self.o0, self.op = self.c_get_dc()
        self.v00 = self.v0
        self.x0 = self.x
        self.c_set_state(self.v0, self.x)
        self.c_calc_pot_update(np.array([self.pot[v] for v in self.pot_list], dtype=self.dtp))
        self.eval = self.eval_c
        self.nonlin = self.nonlin_c

    def set_variable(self, var, val):
        assert var in self.pot
        self.pot[var] = val
        self.c_calc_pot_update(np.array([self.pot[v] for v in self.pot_list], dtype=self.dtp))

    def nonlin_c(self, p):#, K, Hc): ##FIXME
        return self.c_calc_nonlin(p, self.v0) ##FIXME

    def eval_c(self, v_in, ii=-1):
        v_in = np.array(v_in)
        if v_in.ndim == 1:
            v_in = v_in.reshape((len(v_in),1))
        assert v_in.shape[1] == self.ni
        y = np.empty((v_in.shape[0], self.no))
        x = self.x
        v = self.v0
        t1 = time.time()
        for n, u in enumerate(v_in):
            y[n,:], x, v = self.c_calc(u, x, v)
        self.time_per_sample = (time.time() - t1)/n
        self.x = x
        self.v0 = v
        return y

    def reset(self):
        self.v0 = self.v00
        self.x = self.x0
        self.c_set_state(self.v00, self.x0)

    def eval_c(self, v_in, ii=-1):
        self.c_set_state(self.v0, self.x)
        t1 = time.time()
        try:
            return self.c_calc_stream(v_in, ii)
        finally:
            self.time_per_sample = (time.time() - t1)/v_in.shape[0]
            self.v0, self.x, self.minmax, info, nfev, fnorm = self.c_get_info()

    def load_from_shared_lib(self):
        INTERFACE_VERSION = 5
        try:
            lib = ct.cdll.LoadLibrary(self.soname)
        except OSError as e:
            raise SystemExit(e)
        try:
            version = lib.get_interface_version()
        except AttributeError:
            raise SystemExit("%s: bad shared lib, missing get_interface_version" % self.soname)
        if version != INTERFACE_VERSION:
            raise SystemExit("interface version %d expected (found: %d)" % (INTERFACE_VERSION, version))
        c_char_pp = ct.POINTER(ct.c_char_p)
        c_get_structure = lib.get_structure
        c_get_structure.restype = None
        c_get_structure.argtypes = [c_char_pp, ct.POINTER(ct.c_int), ct.POINTER(ct.c_int), ct.POINTER(ct.POINTER(ct.c_int)),
                                    ct.POINTER(ct.POINTER(ct.c_int)), ct.POINTER(c_char_pp), ct.POINTER(c_char_pp),
                                    c_char_pp, ct.POINTER(c_char_pp), ct.POINTER(ct.POINTER(ct.c_double)),
                                    ct.POINTER(c_char_pp), c_char_pp]
        nm = ct.c_char_p()
        sz = ct.c_int()
        fs = ct.c_int()
        t = ct.POINTER(ct.c_int)()
        tc = ct.POINTER(ct.c_int)()
        pins = c_char_pp()
        cnm = c_char_pp()
        p = ct.c_char_p()
        plist = c_char_pp()
        pvals = ct.POINTER(ct.c_double)()
        ol = c_char_pp()
        cmt = ct.c_char_p()
        c_get_structure(ct.byref(nm), ct.byref(sz), ct.byref(fs), ct.byref(t), ct.byref(tc), ct.byref(pins),
                        ct.byref(cnm), ct.byref(p), ct.byref(plist), ct.byref(pvals), ct.byref(ol),
                        ct.byref(cmt))
        nx, ni, no, npl, nn, nni, nno, nc, end = [t[i] for i in range(9)]
        if end != -1:
            raise SystemExit("%s: bad sequence length in get_structure")
        self.nx, self.ni, self.no, self.npl, self.nn, self.nni, self.nno = nx, ni, no, npl, nn, nni, nno
        self.comp_sz = comp_sz = np.array([[slice(tc[6*i+2*j],tc[6*i+2*j+1]) for j in range(3)] for i in range(nc)])
        self.pins = np.array([pins[i] for i in range(nn)])
        self.comp_names = [cnm[2*i] for i in range(nc)]
        self.comp_namespace = [cnm[2*i+1] for i in range(nc)]
        self.name = nm.value
        self.data_size = sz.value
        self.fs = fs.value
        self.method = p.value
        self.pot_list = [plist[i] for i in range(npl)]
        self.pot = dict([(plist[i], pvals[i]) for i in range(npl)])
        self.out_labels = [ol[i] for i in range(no)]
        self.comment = cmt.value
        if self.data_size == 8:
            c_real = ct.c_double
            self.dtp = np.float64
        elif self.data_size == 4:
            c_real = ct.c_float
            self.dtp = np.float32
        else:
            raise ValueError("unknown data size: %d" % self.data_size)
        def c_arr(n, w=False):
            flags = ['C']
            if w:
                flags.append('W')
            return np.ctypeslib.ndpointer(dtype=c_real, ndim=1, shape=(n,), flags=flags)
        def c_mat(w=False):
            flags = ['C']
            if w:
                flags.append('W')
            return np.ctypeslib.ndpointer(dtype=c_real, ndim=2, flags=flags)
        c_int_p = ct.POINTER(ct.c_int)
        c_real_p = ct.POINTER(c_real)
        c_calc = lib.calc
        c_calc.restype = ct.c_int
        c_calc.argtypes = [c_arr(ni), c_arr(nx), c_arr(nn,True), c_arr(nx,True), c_arr(no,True), c_int_p, c_int_p, c_real_p]
        c_set_state = lib.set_state
        c_set_state.restype = None
        c_set_state.argtypes = [c_arr(nn), c_arr(nx)]
        c_get_info = lib.get_info
        c_get_info.restype = None
        c_get_info.argtypes = [c_arr(nn,True), c_arr(nx, True), c_arr(nni,True), c_arr(nni,True), c_int_p, c_int_p, c_real_p]
        c_calc_stream = lib.calc_stream
        c_calc_stream.restype = ct.c_int
        c_calc_stream.argtypes = [c_mat(), c_mat(True), ct.c_int]
        info = ct.c_int()
        nfev = ct.c_int()
        fnorm = c_real()
        if nn:
            c_calc_nonlin = lib.calc_nonlin
            c_calc_nonlin.restype = ct.c_int
            c_calc_nonlin.argtypes = [ct.c_int, c_mat(), c_mat(True), c_arr(nn, True), c_int_p, c_int_p, c_real_p]
            def calc_nonlin(p, v):
                assert p.shape[1] == nni+npl, (p.shape[1], nni+npl)
                p = np.array(p, dtype=self.dtp, order='C', copy=False)
                i = np.zeros((p.shape[0], nno), dtype=self.dtp, order='C')
                r = c_calc_nonlin(len(p), p, i, v, ct.byref(info), ct.byref(nfev), ct.byref(fnorm))
                if r != 0:
                    raise ValueError("convergence error: info=%d, nfev=%d, fnorm=%g" % (info.value, nfev.value, fnorm.value))
                return i
            self.c_calc_nonlin = calc_nonlin
            def define_func(f, i, npl):
                v_slice, p_slice, i_slice = comp_sz[i]
                nni = p_slice.stop - p_slice.start
                nno = i_slice.stop - i_slice.start
                dtp = self.dtp
                def func(p, v=None):
                    assert p.shape[1] == nni+npl, (p.shape[1], nni+npl)
                    if v is None:
                        v = self.v0
                    p = np.array(p, dtype=dtp, order='C', copy=False)
                    i = np.zeros((p.shape[0], nno), dtype=dtp, order='C')
                    r = f(len(p), p, i, v, ct.byref(info), ct.byref(nfev), ct.byref(fnorm))
                    if r != 0:
                        raise ValueError("convergence error: info=%d, nfev=%d, fnorm=%g" % (info.value, nfev.value, fnorm.value))
                    return i
                func.name = self.comp_names[i]
                func.pins = self.pins[v_slice]
                func.v_slice = v_slice
                func.p_slice = p_slice
                func.i_slice = i_slice
                return func
            self.c_calc_comp = []
            for i in range(nc):
                t = getattr(lib, "calc_%s" % self.comp_namespace[i])
                t.restype = ct.c_int
                t.argtypes = [ct.c_int, c_mat(), c_mat(True), c_arr(nn, True), c_int_p, c_int_p, c_real_p]
                self.c_calc_comp.append(define_func(t, i, npl))
        c_calc_pot_update = lib.calc_inv_update
        c_calc_pot_update.restype = None
        c_calc_pot_update.argtypes = [c_arr(npl)]
        c_get_dc = lib.get_dc
        c_get_dc.restype = None
        c_get_dc.argtypes = [c_arr(nn, True), c_arr(nx, True), c_arr(nni, True), c_arr(no, True), c_arr(ni, True)]
        def calc(u, x, v):
            u = np.array(u, dtype=self.dtp)
            x = np.array(x, dtype=self.dtp)
            v = np.array(v, dtype=self.dtp)
            x_new = np.zeros(nx, dtype=self.dtp)
            o = np.zeros(no, dtype=self.dtp)
            if c_calc(u, x, v, x_new, o, ct.byref(info), ct.byref(nfev), ct.byref(fnorm)) < 0:
                if fnorm.value > 1e-7:
                    raise RuntimeError("convergence: method=%s, info=%d, nfev=%d, fnorm=%g"
                                       % (self.method, info.value, nfev.value, fnorm.value))
            return o, x_new, v
        def calc_stream(u, ii=-1):
            assert u.shape[1] == ni + (ii >= 0)
            u = np.array(u, dtype=self.dtp, order="C", copy=False)
            o = np.zeros((u.shape[0], no), dtype=self.dtp)
            if c_calc_stream(u, o, u.shape[0], ii) != 0:
                v, x, minmax, g_info, g_nfev, g_fnorm = get_info()
                raise ValueError("convergence error: info=%d, nfev=%d, fnorm=%g" % (g_info, g_nfev, g_fnorm))
            return o
        def get_info():
            v = np.zeros(nn, dtype=self.dtp, order='C')
            x = np.zeros(nx, dtype=self.dtp, order='C')
            minv = np.zeros(nni, dtype=self.dtp, order='C')
            maxv = np.zeros(nni, dtype=self.dtp, order='C')
            c_get_info(v, x, minv, maxv, ct.byref(info), ct.byref(nfev), ct.byref(fnorm))
            return v, x, np.vstack((minv, maxv)).T, info.value, nfev.value, fnorm.value
        def set_state(v, x):
            v = np.array(v, dtype=self.dtp)
            x = np.array(x, dtype=self.dtp)
            c_set_state(v, x)
        def get_dc():
            v0 = np.zeros(nn, dtype=self.dtp, order='C')
            x0 = np.zeros(nx, dtype=self.dtp, order='C')
            p0 = np.zeros(nni, dtype=self.dtp, order='C')
            o0 = np.zeros(no, dtype=self.dtp, order='C')
            op = np.zeros(ni, dtype=self.dtp, order='C')
            c_get_dc(v0, x0, p0, o0, op)
            return v0, x0, ml.matrix(p0).T, o0, op
        self.c_calc = calc
        self.c_set_state = set_state
        self.c_get_info = get_info
        self.c_calc_stream = calc_stream
        self.c_calc_pot_update = c_calc_pot_update
        self.c_get_dc = get_dc

    def __call__(self, v_in, ii=-1):
        return self.eval_c(v_in, ii)


class Parser(object):
    # Nl = incidence matrix denoting the nodes which potentials are controlling the currents
    # Nr = incidence matrix denoting the nodes where the current flows in (> 0) or out (< 0)
    #
    def __init__(self, S, V, fs, TR=True, create_filter=False, symbolic=False):
        self.fs = fs
        self.TR = TR   # True: TR (trapezoidal) integration, False: BE backward euler
        self.create_filter = create_filter
        self.symbolic = symbolic
        self.mm = 2.0 if TR else 1.0
        self.update(S, V)

    def update(self, S, V):
        self.V = V
        self.nodes = {}
        self.element_name = collections.defaultdict(list)
        tc = self.collect(S, V)
        n = len(self.nodes) + tc["V"]
        self.S = ml.zeros((n,n))
        self.N = dict([(t, ml.zeros((tc[t[0]], n)))
                       for t in "R","Xl","Xr","Nl","Nr","I","O","P"])
        self.Pv = np.zeros(tc["P"])
        self.pot_func = np.array((None,)*tc["P"])
        self.ConstVoltages = ml.zeros(n)
        self.Z = np.zeros(tc["X"])
        self.CZ = np.ones(tc["N"], dtype=int)
        self.f = np.array((None,)*tc["N"])
        if self.create_filter or self.symbolic:
            alpha = sp.symbols("s")
            self.S = sp.Matrix(self.S)
            for k in self.N.keys():
                self.N[k] = sp.Matrix(self.N[k])
            if self.symbolic:
                self.Pv = sp.Matrix(self.Pv)
                d = {}
                for k, v in V.items():
                    if isinstance(k, Node):
                        sym = sp.Symbol(str(k))
                        if isinstance(v, dict):
                            v = v.copy()
                            v["value"] = sym
                        else:
                            v = sym
                        d[k] = v
                V = d
        else:
            alpha = self.mm * self.fs
        def map_conn(c):
            if isinstance(c, Out):
                return c
            if c is None:
                return None
            return self.nodes.get(c, -1)
        for row in S:
            row[0].process(self, [map_conn(c) for c in row[1:]], V.get(row[0]), alpha)
        self.pot = V.get("POT", {})
        self.op = V.get("OP",[0.]*tc["I"])
        self.pot_list = []
        for a, f in self.pot_func:
            s = str(a)
            if s not in self.pot_list:
                self.pot_list.append(s)

    def get_status(self):
        return ""
        
    def matches(self, create_filter, symbolic):
        return self.create_filter == create_filter and self.symbolic == symbolic

    def get_nonlin_funcs(self):
        return self.f

    def set_function(self, idx, expr, vl, base):
        assert self.f[idx] is None
        self.f[idx] = (expr, vl, range(base, base+len(vl)))

    def get_pot_funcs(self):
        return self.pot_func

    def get_pot_attr(self):
        attrlist = []
        for e in set([e[0] for e in self.element_name["P"]]):
            t = self.V[e]
            if not isinstance(t, dict):
                t = dict(value=t)
            var = t.get('var')
            if var is None:
                var = str(e)+"v"
            name = t.get('name', var)
            loga = t.get('a', 0)
            inv = t.get('inv', 0)
            if loga:
                if inv:
                    expr = lambda a: (math.exp(loga * (1 - a)) - 1) / (math.exp(loga) - 1)
                else:
                    expr = lambda a: (math.exp(loga * a) - 1) / (math.exp(loga) - 1)
            else:
                if inv:
                    expr = lambda a: 1-a
                else:
                    expr = lambda a: a
            attrlist.append((var, name, loga, inv, expr))
        return attrlist

    def get_variable_defaults(self):
        return dict([(a, self.pot.get(str(a), 0.5)) for a, f in self.pot_func])

    def extra_variable_index(self, idx):
        return len(self.nodes) + idx

    def extra_variable_by_name(self, tpl):
        try:
            return len(self.nodes) + self.element_name["V"].index(tpl)
        except ValueError:
            print "%s not in %s" % (tpl, self.element_name["V"])
            raise

    def collect(self, S, V):
        tc = collections.Counter()
        l = self.element_name["S"]
        for row in S:
            e = row[0]
            conn = row[1:]
            e.add_count(tc, conn, V.get(e))
            for s in conn:
                if s not in self.nodes:
                    if s != GND and s is not None and not isinstance(s, Out):
                        self.nodes[s] = len(self.nodes)
                        l.append((s,None))
        return tc

    def new_row(self, N, sym, f=None):
        l = self.element_name[N]
        n = len(l)
        if N == "V":
            n += len(self.nodes)
        l.append((sym,f))
        return n

    def current_row(self, N):
        n = len(self.element_name[N]) - 1
        if N == "V":
            n += len(self.nodes)
        return n

    def add_currents(self, m, conn, value):
        if conn[0] != -1:
            m[conn[0], conn[0]] += value
            if conn[1] != -1:
                m[conn[0], conn[1]] += -value
        if conn[1] != -1:
            m[conn[1], conn[1]] += value
            if conn[0] != -1:
                m[conn[1], conn[0]] += -value

    def add_S_currents(self, conn, value):
        self.add_currents(self.S, conn, value)

    def add_S(self, idx, conn, value):
        for i in conn[0], conn[1]:
            if i != -1:
                self.S[idx,i] += value
            value = -value
        for i in conn[0], conn[1]:
            value = -value
            if i != -1:
                self.S[i,idx] += value

    def add_conn(self, N, node, conn, val):
        if conn != -1:
            self.N[N][node, conn] += val

    def add_2conn(self, N, node, conn, value=1):
        if len(conn) != 2:
            raise ValueError("2 connections expected")
        for i, v in zip(conn, (value, -value)):
            self.add_conn(N, node, i, v)

    @staticmethod
    def format_element(el, pref=""):
        n, d = el
        return "%s%s%s" % (pref, n, d and ("[%s]" % d) or "")

    def node_names(self):
        return [self.format_element(v) for v in self.element_name["S"]+self.element_name["V"]]

    def out_labels(self):
        return [str(v) for v in self.element_name["O"][0][0].conn]

    def generate_c_code(self, d):
        # UNUSED
        def mk_sym(s, n):
            return sp.symbols(['%s[%d]' % (s, i) for i in range(len(self.element_name[n]))])
        def ccode(d, ret, ex):
            r = ret + '[%d]'
            d[ret] = "\n".join([sp.ccode(ee, r % i) for i, ee in enumerate(ex)]).replace("\n","\n    ")
        def ccodesum(d, ret, l):
            l2 = [m for m in l if m]
            if l2:
                ccode(d, ret, reduce(operator.add, l2))
            else:
                d[ret] = '/* no %s */' % ret
        x = sp.Matrix(mk_sym('x', 'X'))
        u = sp.Matrix(mk_sym('u', 'I'))
        i = sp.Matrix(mk_sym('i', 'N'))
        v = mk_sym('v', 'N')
        p = mk_sym('p', 'N')
        # nonlinear function for root-finding
        l = []
        for n, (expr, vl, base) in enumerate(self.eq.f):
            for j, e in enumerate(vl):
                expr = expr.subs(e, v[base+j])
            l.append(expr)
        ccode(d, 'fvec', sp.Matrix(p) + sp.Matrix(self.K) * sp.Matrix(l) - sp.Matrix(np.diag(self.eq.CZ)) * sp.Matrix(v))
        # "currents"
        ccode(d, 'i', l)
        # p value
        ccodesum(d, 'p', [sp.Matrix(self.G) * x, sp.Matrix(self.H) * u, sp.Matrix(self.Hc)])
        # x update
        ccodesum(d, 'x_new', [sp.Matrix(self.A) * x, sp.Matrix(self.B) * u, sp.Matrix(self.Bc), sp.Matrix(self.C) * i])
        # output
        ccodesum(d, 'o', [sp.Matrix(self.D) * x, sp.Matrix(self.E) * u, sp.Matrix(self.Ec), sp.Matrix(self.F) * i])


def get_py_executor(parser, solver=None, linearize=False):
    sim = SimulatePy(EquationSystem(parser), solver)
    if linearize:
        J = sim.jacobi()
        sim = SimulatePy(EquationSystem(parser, J), solver)
    return sim

def get_executor(name, parser, solver=None, pure_python=True, c_tempdir=None, c_verbose=False,
                 c_debug_load="", c_real="double", extra_sources=None, linearize=False):
    if pure_python:
        return get_py_executor(parser, solver, linearize)
    elif c_debug_load:
        sim = SimulateC(c_debug_load)
        print "%s/%s: %s[%d], %s, %s" % (sim.name, sim.comment, sim.method, sim.data_size, sim.out_labels, sim.pot_list)
        return sim
    else:
        sim = SimulatePy(EquationSystem(parser), solver)
        return BuildCModule(name, sim, solver, c_tempdir, c_verbose, c_real, extra_sources, linearize).get_executor()[1]



#
# Copyright (C) 2009, 2010 Hermann Meyer, James Warden, Andreas Degert
# Copyright (C) 2011 Pete Shorthose
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
""" faust dsp component wrapped in a python class
"""
import numpy as np
cimport numpy as np
from cython.operator cimport dereference as deref, preincrement as inc

cdef extern from "time.h":
    cdef struct timespec:
        int tv_sec
        int tv_nsec
    ctypedef int clockid_t
    int CLOCK_MONOTONIC
    int clock_gettime(clockid_t clk_id, timespec *tp)

cdef extern from "alloca.h":
    void *alloca(size_t size)

cdef extern from "faustwrap.cpp":
    cppclass param:
        float* fZone
        float fMin
        float fMax

    cppclass string:
        char *c_str()

    cppclass fKeyType
    cppclass fKeyIter:
        fKeyIter operator++()
        bint operator!=(fKeyIter)
        bint operator==(fKeyIter)
        fKeyType operator*()

    cppclass fKeyType:
        string first
        param second
        fKeyIter begin()
        fKeyIter end()
        fKeyIter find(char *s)

    cppclass UI:
        pass

    cppclass mydsp:
        void buildUserInterface(UI* interface)
        int getNumInputs()
        int getNumOutputs()
        void init(int samplingRate)
        void compute(int len, float** inputs, float** outputs)

    cppclass CMDUI(UI):
        fKeyType fKeyParam

ctypedef np.float32_t f32_t
ctypedef float *floatp

cdef inline double ts_diff(timespec ts1, timespec ts2):
    cdef double df = ts1.tv_sec - ts2.tv_sec
    return df * 1e9 + (ts1.tv_nsec - ts2.tv_nsec)

cdef class dsp(object):
    """python class wrapping a faust component

    use keys() to list parameter names

    use get_range to query the allowed parameter range

    use parameter names as index to get/set the value
        parameters are initialized with their default value

    use num_inputs / num_outputs to find out about channel cound

    use init to set the sampling rate (don't forget!)

    use compute to run the dsp algorithm

    use property default_size to set the output size of compute
        or call with an array with requested size

    use nanosec_per_sample to get the last execution time
    """
    cdef mydsp *Cdsp
    cdef CMDUI *interface
    cdef dict Cparameter
    cdef int defsize
    cdef double time_per_sample

    def __cinit__(self):
        self.Cdsp = new mydsp()
        self.interface = new CMDUI()
        self.Cdsp.buildUserInterface(self.interface)
        cdef fKeyIter i
        i = self.interface.fKeyParam.begin()
        p = {}
        while i != self.interface.fKeyParam.end():
            p[deref(i).first.c_str()] = (deref(i).second.fMin, deref(i).second.fMax)
            inc(i)
        self.Cparameter = p
        self.defsize = 128
        self.time_per_sample = 0

    def __dealloc__(self):
        del self.Cdsp
        del self.interface

    property nanosec_per_sample:
        "time in nanoseconds measured for last compute call"
        def __get__(self):
            return self.time_per_sample

    property default_size:
        "default output size of compute call without arguments"
        def __get__(self):
            return self.defsize
        def __set__(self, int n):
            self.defsize = n

    def keys(self):
        "returns list of dsp parameter names"
        return self.Cparameter.keys()

    def get_range(self, key):
        """argument: parameter name
        returns: allowed range tuple (lower, upper)"""
        return self.Cparameter[key]

    def __getitem__(self, char* pname):
        cdef fKeyIter p = self.interface.fKeyParam.find(pname)
        if p == self.interface.fKeyParam.end():
            raise KeyError("not found: %s" % pname)
        return deref(p).second.fZone[0]
    
    def __setitem__(self, char* pname, float pval):
        cdef fKeyIter p = self.interface.fKeyParam.find(pname)
        if p == self.interface.fKeyParam.end():
            raise KeyError("not found: %s" % pname)
        if not (deref(p).second.fMin <= pval <= deref(p).second.fMax):
            raise ValueError(
                "parameter %s value %s outside range [%s, %s]"
                % (pname, pval, deref(p).second.fMin, deref(p).second.fMax))
        deref(p).second.fZone[0] = pval

    property num_inputs:
        "number of input channels"
        def __get__(self):
            return self.Cdsp.getNumInputs()

    property num_outputs:
        "number of output channels"
        def __get__(self):
            return self.Cdsp.getNumOutputs()

    def init(self, int samplingRate):
        """argument: sample rate
        must be called before calling compute"""
        self.Cdsp.init(samplingRate)

    def compute(self, np.ndarray inp = None):
        """execute dsp algorithm

        argument: 2-dim float32 numpy array with at least num_inputs rows
        returns: numpy float32 array with num_outputs rows

        when num_inputs == 0 the output will be size default_size (can be set)
        or call with an array of requested size

        when num_inputs == 1 you can use a 1-dim array

        specify type when creating an array, e.g. zeros(200, dtype=float32)
        """
        cdef int ni = self.Cdsp.getNumInputs()
        cdef int count
        if ni == 0 and inp is None:
            count = self.defsize
        else:
            if not isinstance(inp, np.ndarray):
                raise ValueError("need ndarray")
            if inp.dtype != np.float32:
                raise ValueError("need float32")
            count = inp.shape[inp.ndim-1]
        if ni == 1:
            if not (inp.ndim == 1 or (inp.ndim == 2 and inp.shape[1] >= 1)):
                raise ValueError("need vector or 2-dim array with 1 row")
        elif ni > 1:
            if not (inp.ndim == 2 and inp.shape[1] >= ni):
                raise ValueError("need 2-dim array with at least %d rows" % ni)
        cdef floatp *ina = <floatp*>alloca(ni*sizeof(floatp))
        cdef int i, n
        if inp is not None:
            n = inp.strides[0]
            for i in range(ni):
                ina[i] = <floatp>(inp.data+i*n)
        cdef int no = self.Cdsp.getNumOutputs()
        cdef floatp *oa = <floatp*>alloca(no*sizeof(floatp))
        cdef np.ndarray o
        if no == 1:
            o = np.empty(count,dtype=np.float32)
        else:
            o = np.empty((no,count),dtype=np.float32)
        n = o.strides[0]
        for i in range(no):
            oa[i] = <floatp>(o.data + i*n)
        cdef timespec t0, t1
        clock_gettime(CLOCK_MONOTONIC, &t0)
        self.Cdsp.compute(count, ina, oa)
        clock_gettime(CLOCK_MONOTONIC, &t1)
        self.time_per_sample = ts_diff(t1,t0)/count
        return o



import os
from analog import *

schema = "Fuzzface2.sch"
path = "tmp"
module_id = "fuzzface"
mod = os.path.join(path, module_id+".so")


# create plugin
c1 = Circuit()
c1.plugindef = dk_simulator.PluginDef(module_id)
c1.plugindef.name = "Fuzz Face"
c1.plugindef.description = "J Hendrix Fuzz Face simulation"
c1.plugindef.category = "Distortion"
c1.plugindef.id = "fuzzface"
c1.set_module_id(module_id)
c1.read_gschem(schema)
c1.create_faust_module()



from __future__ import division
import sympy, numpy, types, math
import dk_lib
import pylab, scipy.signal

class GeneratedSignal(object):

    def __init__(self, func, op, fs, freq, start_freq, stop_freq):
        self.op = op
        self.fs = fs
        self.freq = freq
        self.start_freq = start_freq
        self.stop_freq = stop_freq
        self.generate_spectrum = False
        self.generate_harmonics = False
        self.make_spectrum = self._default_make_spectrum
        m = [dict(sweep = self._sweep, chirp = self._chirp, impulse = self._impulse, time = self._time,
                  null=self._null, asin=numpy.arcsin, atan=numpy.arctan,
                  Min=numpy.minimum, Max=numpy.maximum, ramp=self._ramp), numpy]
        self.signal = sympy.lambdify((), func, modules=m)()
        if isinstance(self.signal, tuple):
            self.signal = numpy.array(self.signal).T
        else:
            self.signal = self.signal[:,numpy.newaxis]
        samples = len(self.signal)
        if len(op) != self.signal.shape[1]:
            raise ValueError(
                "signal definition error: inconsistent channel count (%d vs. %d in OP definition)"
                % (self.signal.shape[1], len(op)))
        self.input_signal = self.signal + numpy.array(op, dtype=numpy.float64)
        self.timeline = numpy.linspace(0, samples/fs, samples, endpoint=False)

    def _sweep_make_spectrum(self, response, freqlist, shift=True):
        return self._sweep_harmonics_responses(response, 1, freqlist, shift)[0]

    @staticmethod
    def _fft_convolve(h, xd):
        n = len(h) + len(xd) - 1
        np = dk_lib.pow2roundup(n)
        if len(xd.shape) == 2 and len(h.shape) == 1:
            h = h.reshape((len(h), 1))
        s = numpy.fft.irfft(numpy.fft.rfft(h, np, axis=0) * numpy.fft.rfft(xd, np, axis=0), np, axis=0)
        return s[:n]

    def _sweep_harmonics_responses(self, response, N, freqlist, shift=True):
        g = self.input_signal.ptp()/2
        di = self._fft_convolve(self.sweep_inverse_signal, response/g)  # deconvolved impulse response
        imp_indices = numpy.zeros(N+1, dtype=int)  # the indices of each impulse (1st linear, 2nd the first harm. dist. etc.)
        imp_indices[0] = len(self.sweep_inverse_signal) - 1
        for n in range(1, N+1):
            imp_indices[n] = imp_indices[0] - int(round(math.log(n+1) * self.sweep_rate))
        imps = []
        for n in range(N):
            dl = (imp_indices[n] - imp_indices[n+1]) // 2
            lo = imp_indices[n] - dl  # the low limit where to cut
            hi = imp_indices[n] + dl  # the high limit where to cut
            di_w = di[lo:hi]
            win = numpy.blackman(len(di_w))
            if len(di.shape) == 2:
                win = win.reshape((len(win), 1))
            di_w *= win  # smoothed version
            w = freqlist
            if shift:
                w = w * (n + 1)
            h = scipy.signal.freqz(di_w, worN=w)[1]
            off = imp_indices[n] - lo
            h *= numpy.exp(off * 1j * freqlist)
            imps.append(h)
        return imps

    def _sweep(self, t, start, stop, pre, post, fs):
        self.generate_spectrum = True
        self.generate_harmonics = True
        self.make_spectrum = self._sweep_make_spectrum
        self.make_harmonics_responses = self._sweep_harmonics_responses
        post = int(round(post * fs))
        if pre < 0:
            pre = int(round((len(t) - post) * 0.1))
            if pre < 0:
                pre = 0
        else:
            pre = int(round(pre * fs))
        if post >= len(t) - pre:
            post = int((len(t) - pre) * 0.1)
        span = len(t) - (pre + post)
        self.pre = pre
        self.post = post
        self.fs = fs
        sig, self.sweep_inverse_signal, self.start, self.stop, self.sweep_rate = \
          dk_lib.genlogsweep(start, stop, fs, pre, span, post)
        return sig

    def _chirp(self, t, start, stop, pre, post, fs):
        self.generate_spectrum = True
        self.generate_harmonics = True
        self.make_spectrum = self._sweep_make_spectrum
        self.make_harmonics_responses = self._sweep_harmonics_responses
        post = int(round(post * fs))
        if pre < 0:
            pre = int(round((len(t) - post) * 0.1))
        else:
            pre = int(round(pre * fs))
        span = len(t) - (pre + post)
        self.pre = pre
        self.post = post
        self.fs = fs
        sig, self.sweep_inverse_signal, self.start, self.stop, self.sweep_rate = \
          dk_lib.expchirp(start, stop, fs, pre, span, post)
        return sig

    def _impulse_make_spectrum(self, response, freqlist):
        g = self.input_signal.ptp()
        return scipy.signal.freqz(response/g, worN=freqlist)[1]

    def _impulse(self, t, offset):
        self.generate_spectrum = True
        self.make_spectrum = self._impulse_make_spectrum
        a = numpy.zeros_like(t)
        a[offset] = 1
        return a

    def _time(self, samples, fs):
        return numpy.linspace(0, samples/fs, samples)

    def _ramp(self, t, start, log):
        if log:
            if start < 0:
                start = 0.001
            return numpy.logspace(numpy.log10(start), 0, len(t))
        else:
            if start < 0:
                start = 0
            return numpy.linspace(start, 1, len(t))

    def _null(self, s):
        return 0*s

    def _default_make_spectrum(self, response, freqlist):
        n = dk_lib.pow2roundup(len(response))
        return numpy.fft.rfft(response, n, axis=0)

    def has_spectrum(self):
        return self.generate_spectrum

    def get_spectrum(self, response, freqlist, **kw):
        return self.make_spectrum(response, freqlist, **kw)

    def has_harmonics(self):
        return self.generate_harmonics

    def get_harmonics_responses(self, response, N, freqlist, shift=True):
        return self.make_harmonics_responses(response, N, freqlist, shift)

    def _plot_setup(self, lower_freq, upper_freq, plotfunc, freqlist):
        if lower_freq is None:
            lower_freq = self.start_freq
        if upper_freq is None:
            upper_freq = self.stop_freq
        if plotfunc is None:
            plotfunc = pylab.semilogx
        if isinstance(freqlist, int):
            freqlist = numpy.logspace(numpy.log10(lower_freq), numpy.log10(upper_freq), freqlist)
        return lower_freq, upper_freq, plotfunc, freqlist

    def plot_spectrum(self, response, lower_freq=None, upper_freq=None, plotfunc=None, freqlist=200):
        lower_freq, upper_freq, plotfunc, freqlist = self._plot_setup(lower_freq, upper_freq, plotfunc, freqlist)
        h = self.get_spectrum(response, 2 * numpy.pi * freqlist / self.fs)
        return plotfunc(freqlist, 20 * numpy.log10(abs(h)))

    def plot_harmonic_spectrum(self, response, nharmonics=6, lower_freq=None, upper_freq=None, plotfunc=None, freqlist=200):
        lower_freq, upper_freq, plotfunc, freqlist = self._plot_setup(lower_freq, upper_freq, plotfunc, freqlist)
        lines = []
        for i, h in enumerate(self.get_harmonics_responses(response, nharmonics, 2*numpy.pi*freqlist/self.fs)):
            if h.size:
                lines.extend(plotfunc(freqlist, 20 * numpy.log10(abs(h))))
        return lines

    def plot(self, response, label=None, clip=-80, nharmonics=8, spectrum=None, freq_range=None):
        if freq_range is not None:
            lower_freq, upper_freq = freq_range
        else:
            lower_freq = upper_freq = None
        if lower_freq is None:
            lower_freq = self.start_freq
        if upper_freq is None:
            upper_freq = self.stop_freq
        if  self.has_harmonics() and nharmonics > 1 and (spectrum is None or spectrum):
            lines = self.plot_harmonic_spectrum(response, nharmonics=nharmonics, lower_freq=lower_freq, upper_freq=upper_freq)
            pylab.xlim(left=lower_freq, right=upper_freq)
            for i, line in enumerate(lines):
                if label:
                    line.set_label("%d, %s" % (i+1, label))
                else:
                    line.set_label("%d" % (i+1))
            return lines
        elif (spectrum is None and self.has_spectrum()) or spectrum:
            f = numpy.logspace(numpy.log10(lower_freq), numpy.log10(upper_freq), 200)
            h = self.get_spectrum(response, 2*numpy.pi * f / self.fs)
            s = 20*numpy.log10(abs(h))
            lines = pylab.plot(f, numpy.where(s > clip, s, numpy.nan), label=label)
            pylab.xscale('log')
            return lines
        else:
            return pylab.plot(self.timeline, response, label=label)


class Signal(object):

    _s_FS, _s_freq, _s_start_freq, _s_stop_freq, _s_sweep_pre, _s_sweep_post, _s_samples = sympy.symbols(
        "FS, freq, start_freq, stop_freq, sweep_pre, sweep_post, samples")
    _s_sweep, _s_chirp, _s_impulse, _s_time, _s_null, _s_ramp = sympy.symbols(
        "sweep,chirp,impulse,time,null,ramp", cls=sympy.Function)
    t = sympy.symbols("t")

    def __init__(self, timespan=0.01):
        self._freq = 440
        self._start_freq = 20
        self._stop_freq = 10000
        self._timespan = timespan
        self._sweep_pre = -1
        self._sweep_post = 0.1
        self.impulse_offset = 0

    @property
    def freq(self):
        return self._freq
    @freq.setter
    def freq(self, v):
        self._freq = v

    @property
    def start_freq(self):
        return self._start_freq
    @start_freq.setter
    def start_freq(self, v):
        self._start_freq = v

    @property
    def stop_freq(self):
        return self._stop_freq
    @stop_freq.setter
    def stop_freq(self, v):
        self._stop_freq = v

    @property
    def timespan(self):
        return self._timespan
    @timespan.setter
    def timespan(self, v):
        self._timespan = v

    @property
    def sweep_pre(self):
        return self._sweep_pre
    @sweep_pre.setter
    def sweep_pre(self, v):
        self._sweep_pre = v

    @property
    def sweep_post(self):
        return self._sweep_post
    @sweep_post.setter
    def sweep_post(self, v):
        self._sweep_post = v

    def sine(self, freq=None):
        if freq is None:
            freq = self.freq
        self.freq = freq
        return sympy.sin(self.t * freq * 2 * sympy.pi)

    def triangle(self, freq=None):
        if freq is None:
            freq = self.freq
        return 2 / sympy.pi * sympy.asin(self.sine(freq))

    def sawtooth(self, freq=None):
        if freq is None:
            freq = self.freq
        return -2 / sympy.pi * sympy.atan(1/sympy.tan(sympy.pi * self.t))

    def square(self, freq=None):
        if freq is None:
            freq = self.freq
        return self.triangle(freq).diff()/(4*freq)

    def sweep(self, start_freq=None, stop_freq=None, pre=None, post=None):
        if start_freq is None:
            start_freq = self._s_start_freq
        if stop_freq is None:
            stop_freq = self._s_stop_freq
        if stop_freq is None:
            stop_freq = self._s_stop_freq
        if pre is None:
            pre = self._s_sweep_pre
        if post is None:
            post = self._s_sweep_post
        return self._s_sweep(self.t, start_freq, stop_freq, pre, post, self._s_FS)

    def chirp(self, start_freq=None, stop_freq=None, pre=None, post=None):
        if start_freq is None:
            start_freq = self._s_start_freq
        if stop_freq is None:
            stop_freq = self._s_stop_freq
        if stop_freq is None:
            stop_freq = self._s_stop_freq
        if pre is None:
            pre = self._s_sweep_pre
        if post is None:
            post = self._s_sweep_post
        return self._s_chirp(self.t, start_freq, stop_freq, pre, post, self._s_FS)

    def ramp(self, start=-1, log=False):
        return self._s_ramp(self.t, start, int(log))

    def impulse(self):
        return self._s_impulse(self.t, self.impulse_offset)

    def generate(self, sig, fs, op=None, channels=None):
        if op is None:
            if channels is None:
                op = [0]
            else:
                op = [0] * channels
        d = {
            self._s_FS: fs,
            self._s_freq: self.freq,
            self._s_start_freq: self.start_freq,
            self._s_stop_freq: self.stop_freq,
            self._s_sweep_pre: self.sweep_pre,
            self._s_sweep_post: self.sweep_post,
            }
        return GeneratedSignal(self(sig).subs(d), op, fs, self.freq, self.start_freq, self.stop_freq)

    def __call__(self, expr, samples=None, timespan=None, periods=None):
        if isinstance(expr, types.FunctionType):
            expr = expr()
        if timespan is not None:
            samples = self._s_FS * timespan
        elif periods is not None:
            samples = periods * self._s_FS / self._s_freq ##FIXME
        elif samples is None:
            samples = self._s_FS * self.timespan
        if isinstance(expr, (int,float)) or not (self.t in expr.atoms() or expr.atoms(self._s_time)):
            expr += self._s_null(self.t)
        return expr.subs(self.t, self._s_time(samples, self._s_FS))



from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext

ext_modules = [Extension("pyamp",
                         ["pyamp.pyx"],
                         language="c++",
                         extra_objects=["intpp.o","data.o"],
                         extra_compile_args=['-UNDEBUG'],
                         extra_link_args=['-fPIC','-UNDEBUG'],
                         )]

setup(
    cmdclass = {'build_ext': build_ext},
    ext_modules = ext_modules
    )



import sys
import itertools, fractions
import numpy as np
import pylab
from scipy.interpolate import LSQUnivariateSpline, InterpolatedUnivariateSpline, UnivariateSpline

try:
    np.pad
except AttributeError:
    # copied from newer numpy distribution
    def _create_vector(vector, pad_tuple, before_val, after_val):
        vector[:pad_tuple[0]] = before_val
        if pad_tuple[1] > 0:
            vector[-pad_tuple[1]:] = after_val
        return vector

    def _normalize_shape(narray, shape):
        normshp = None
        shapelen = len(np.shape(narray))
        if (isinstance(shape, int)):
            normshp = ((shape, shape), ) * shapelen
        elif (isinstance(shape, (tuple, list))
                and isinstance(shape[0], (tuple, list))
                and len(shape) == shapelen):
            normshp = shape
            for i in normshp:
                if len(i) != 2:
                    fmt = "Unable to create correctly shaped tuple from %s"
                    raise ValueError(fmt % (normshp,))
        elif (isinstance(shape, (tuple, list))
                and isinstance(shape[0], (int, float, long))
                and len(shape) == 1):
            normshp = ((shape[0], shape[0]), ) * shapelen
        elif (isinstance(shape, (tuple, list))
                and isinstance(shape[0], (int, float, long))
                and len(shape) == 2):
            normshp = (shape, ) * shapelen
        if normshp == None:
            fmt = "Unable to create correctly shaped tuple from %s"
            raise ValueError(fmt % (shape,))
        return normshp

    def _validate_lengths(narray, number_elements):
        shapelen = len(np.shape(narray))
        normshp = _normalize_shape(narray, number_elements)
        for i in normshp:
            if i[0] < 0 or i[1] < 0:
                fmt ="%s cannot contain negative values."
                raise ValueError(fmt % (number_elements,))
        return normshp

    def _constant(vector, pad_tuple, iaxis, kwargs):
        nconstant = kwargs['constant_values'][iaxis]
        return _create_vector(vector, pad_tuple, nconstant[0], nconstant[1])

    def pad(array, pad_width, mode=None, **kwargs):
        narray = np.array(array)
        pad_width = _validate_lengths(narray, pad_width)

        modefunc = {
               'constant': _constant,
               }

        allowedkwargs = {
               'constant': ['constant_values'],
                }

        kwdefaults = {
                'stat_length': None,
                'constant_values': 0,
                'end_values': 0,
                'reflect_type': 'even',
                }

        if isinstance(mode, str):
            function = modefunc[mode]

            # Make sure have allowed kwargs appropriate for mode
            for key in kwargs:
                if key not in allowedkwargs[mode]:
                    raise ValueError('%s keyword not in allowed keywords %s' %
                                      (key, allowedkwargs[mode]))

            # Set kwarg defaults
            for kw in allowedkwargs[mode]:
                kwargs.setdefault(kw, kwdefaults[kw])

            # Need to only normalize particular keywords.
            for i in kwargs:
                if i == 'stat_length' and kwargs[i]:
                    kwargs[i] = _validate_lengths(narray, kwargs[i])
                if i in ['end_values', 'constant_values']:
                    kwargs[i] = _normalize_shape(narray, kwargs[i])
        elif mode == None:
            raise ValueError('Keyword "mode" must be a function or one of %s.' %
                              (modefunc.keys(),))
        else:
            # User supplied function, I hope
            function = mode

        # Create a new padded array
        rank = range(len(narray.shape))
        total_dim_increase = [np.sum(pad_width[i]) for i in rank]
        offset_slices = [slice(pad_width[i][0],
                               pad_width[i][0] + narray.shape[i])
                         for i in rank]
        new_shape = np.array(narray.shape) + total_dim_increase
        newmat = np.zeros(new_shape).astype(narray.dtype)

        # Insert the original array into the padded array
        newmat[offset_slices] = narray

        # This is the core of pad ...
        for iaxis in rank:
            np.apply_along_axis(function,
                                iaxis,
                                newmat,
                                pad_width[iaxis],
                                iaxis,
                                kwargs)
        return newmat

    np.pad = pad

################################################################

class KnotData(object):

    def __init__(self, knots, mapping, sl, order):
        self.knots = knots
        self.mapping = mapping
        self.slice_data = sl
        self.order = order

    def used(self):
        return self.order is not None

    def inv_h(self):
        sl = self.slice_data
        return (sl.step.imag - 1) / (sl.stop - sl.start)

    def count(self):
        return int(self.slice_data.step.imag)

    def get_knots(self):
        k = self.order
        return self.knots[k:-k]

    def num_coeffs(self):
        return len(self.knots) - self.order

    def get_bbox(self):
        return self.knots[0], self.knots[-1]

    def get_order(self):
        return self.order


class TensorSpline(object):

    def __init__(self, func, ranges, basegrid):
        self.func = func
        self.ranges = ranges
        self.basegrid = basegrid
        self.knot_data = np.empty((len(basegrid), len(ranges)), dtype=object)
        self.find_knots()

    def calc_coeffs(self):
        coeff = []
        for v, (g, kdata) in enumerate(zip(self.basegrid, self.knot_data)):
            ranges = [(slice(r[0], r[1], t[0]*1j) if kd.used() else slice((r[1]-r[0])/2.0,r[1],1j)) for r, t, kd in zip(self.ranges,g[0],kdata)]
            fnc = self.calc_grid(ranges, g[1], g[2])[v]
            for n in range(len(ranges)):
                kd = self.knot_data[v,n]
                if not kd.used():
                    continue
                s = list(fnc.shape)
                s[n] = kd.num_coeffs()
                out = np.empty(s)
                s[n] = 1
                x = np.mgrid[ranges[n]]
                for i in np.ndindex(tuple(s)):
                    i = list(i)
                    i[n] = slice(None)
                    ss = LSQUnivariateSpline(x, fnc[i], kd.get_knots(), bbox=kd.get_bbox(), k=kd.get_order()-1)
                    out[i] = ss.get_coeffs()
                fnc = out
            coeff.append(fnc)
        return coeff

    @staticmethod
    def get_interpolating_knots(x, k):
        x_e = x[-1]
        x_e += (x_e - x[-2]) * 0.1
        start = k // 2
        end = -(k - start)
        knots = np.zeros((3,len(x)-k))
        knots[0] = x[start:end]
        knots[1] = np.arange(start, len(x)+end)
        if k % 2:
            knots[0,:-1] += knots[0,1:]
            knots[0,-1] += x[end]
            knots[0] *= 0.5
        return knots, (x[0], x_e)

    @staticmethod
    def spline_diff(n, x, kn, bbox, k, fnc):
        s = list(fnc.shape)
        s[n] = 1
        out = np.empty_like(fnc)
        for idx in np.ndindex(tuple(s)):
            idx = list(idx)
            idx[n] = slice(None)
            f = fnc[idx]
            ss = LSQUnivariateSpline(x, f, kn, bbox=bbox, k=k-1)
            out[idx] = abs(ss(x) - f)
        return out

    @staticmethod
    def max_df(n, x, kn, bbox, k, fnc):
        s = list(fnc.shape)
        s[n] = 1
        mx = 0
        for idx in np.ndindex(tuple(s)):
            idx = list(idx)
            idx[n] = slice(None)
            f = fnc[idx]
            ss = LSQUnivariateSpline(x, f, kn, bbox=bbox, k=k-1)
            mx = max(mx, np.amax(abs(ss(x) - f)))
        return mx #/ np.ptp(fnc)

    def calc_grid(self, ranges, pre, post):
        grd = np.mgrid[ranges]
        pre_grd = grd.copy()
        if pre:
            grd = pre(grd)
        grd_shape = grd.shape
        numpoints = np.product(grd_shape[1:])
        grd = grd.reshape(grd_shape[0], numpoints)
        fnc = self.func(grd.T, True).T
        fnc.shape = (len(self.basegrid),)+tuple(grd_shape[1:])
        if post:
            fnc = post(pre_grd, fnc)
        return fnc

    @staticmethod
    def max_diff(n, kn, pos, val):
        ki = np.unique(np.digitize(pos, kn), True)[1]
        idx = [slice(None)]*val.ndim
        l = []
        for i, j in itertools.izip_longest(ki, ki[1:]):
            idx[n] = slice(i, j)
            l.append(np.amax(val[idx]))
        a = np.array(l)
        return a[1:] + a[:-1]

    @staticmethod
    def mk_result(kn, bbox, k, r):
        idx = np.zeros(len(kn[1])+2, dtype=np.int32)
        idx[1:-1] = kn[1]
        idx[-1] = r.step.imag
        f = reduce(fractions.gcd, idx)
        idx /= f
        a = np.empty(idx[-1], dtype=np.int32)
        for m, (i, j) in enumerate(itertools.izip_longest(idx[:-1], idx[1:])):
            a[i:j] = m
        a += k-1
        return KnotData(np.pad(kn[0], k, 'constant', constant_values=bbox), a, slice(r.start, r.stop, r.step/f), k)

    def get_support(self, i, fnc, dlim):
        def apply_subcube(op, a, axis):
            j = a.shape[axis]
            return op(np.rollaxis(a, axis, 1).reshape(j, a.size/j), axis=1)
        dfnc = np.diff(fnc, 1, axis=i)
        supp1 = apply_subcube(np.amax, abs(dfnc), i) >= dlim
        nz1 = np.flatnonzero(supp1)
        d2fnc = np.diff(dfnc, 1, axis=i)
        d2lim = 1e-7
        supp2 = apply_subcube(np.amax, abs(d2fnc), i) >= d2lim
        nz2 = np.flatnonzero(supp2)
        istart, iend, llim, rlim = None, None, None, None
        if len(nz1):
            istart = nz1[0]
            iend = nz1[-1]
        if len(nz2):
            istart = min(istart, nz2[0])
            iend = max(iend, nz2[-1])
        if istart < 2:
            istart = None
        if iend > d2fnc.shape[i]-1:
            iend = None
        if istart:
            istart -= 1
            if np.amax(apply_subcube(np.amax, abs(dfnc), i)[:istart]) < dlim:
                llim = 0
            else:
                llim = (np.amin(apply_subcube(np.amin, dfnc, i)[:istart]), np.amax(apply_subcube(np.amax, dfnc, i)[:istart]))
        if iend:
            iend += 1
            if np.amax(apply_subcube(np.amax, abs(dfnc), i)[iend:]) < dlim:
                rlim = 0
            else:
                rlim = (np.amin(apply_subcube(np.amin, dfnc, i)[:iend]), np.amax(apply_subcube(np.amax, dfnc, i)[:iend]))
                print apply_subcube(np.amax, abs(dfnc), i)[iend:]; print iend, dlim; raise SystemExit
        print "##", dlim, (istart, iend), (llim, rlim)
        return (istart, iend), (llim, rlim)

    def find_knots(self):
        for v, g in enumerate(self.basegrid):
            order = [t[1] for t in g[0] if t is not None]
            for n in range(len(self.ranges)):
                ranges = [slice(r[0], r[1], (2j * t[0] + 1 if i == n else 0.5j * t[0])) for i, (r, t) in enumerate(zip(self.ranges,g[0]))]
                fnc = self.calc_grid(ranges, g[1], g[2])[v]
                #self.get_support(n, fnc, np.amax(abs(fnc))*(ranges[n].stop-ranges[n].start)/ranges[n].step.imag * 1e-4)
                ptp = fnc.ptp()
                if np.amax(fnc.ptp(axis=n)) < 1e-6 * ptp:
                    print "%s[%d,%d]: const: %g" % (self.func.comp_id, v, n, np.amax(fnc.ptp(axis=n)) / ptp)
                    self.knot_data[v, n] = KnotData(None,None,None,None)
                    continue
                k = order[n]
                x = np.mgrid[ranges[n]]
                rng = slice(self.ranges[n][0], self.ranges[n][1], g[0][n][0]*1j)
                kn, bbox = self.get_interpolating_knots(np.mgrid[rng], k)
                #e_max = 1.1 * self.max_df(n, x, kn, bbox, k, fnc)
                try:
                    e_rel = g[3]
                except IndexError:
                    e_rel = None
                if e_rel is None:
                    #e_rel = 4e-5
                    e_rel = 4e-4
                e_max = ptp * e_rel
                good = kn
                am = None
                kn0 = kn
                try:
                    opt = g[4]
                except IndexError:
                    opt = True
                while opt:
                    df = self.spline_diff(n, x, kn[0], bbox, k, fnc)
                    mx = np.amax(df)
                    if mx > e_max:
                        if am is None:
                            raise ValueError("bad approximation for %s[%d,%d]: %g > %g" % (self.func.comp_id, v, n, mx, e_max))
                        kn = good
                        kn[2,am] = 1
                    else:
                        good = kn
                    if False:
                        xd = np.square(np.diff(ss(kn[:-1], k))).argsort()+1
                    else:
                        xd = self.max_diff(n, np.pad(kn[0],(1,0),'constant',constant_values=(x[0],)), x, df).argsort()
                    for am in xd:
                        if not kn[2,am]:
                            break
                    else:
                        break
                    kn = np.append(kn[:,:am],kn[:,am+1:], axis=1)
                self.knot_data[v, n] = self.mk_result(kn, bbox, k, rng)
                print "%s[%d,%d]: (%g), %d -> %d" % (self.func.comp_id, v, n, e_max, len(kn0[0]), len(kn[0]))
                #print "#", fnc.shape, x.shape, e_max
                #print kn[:1]
                #pylab.plot(x, fnc[:,0,:])
                #pylab.plot(kn, np.zeros(len(kn)),"x")
                #pylab.show(); raise SystemExit


def b_ink(x, fnca, k):
    "return t, bcoef"
    ca = []
    for fnc in fnca:
        t = [None]*len(x)
        for n in range(len(x)):
            cf = np.empty_like(fnc)
            s = list(fnc.shape)
            s[n] = 1
            xx = x[n]
            xxe = xx[-1]
            xxe += (xxe-xx[-2]) * 0.1
            kk = k[n]
            start = kk // 2
            end = -(kk - start)
            knots = xx[start:end].copy()
            if kk % 2:
                knots[:-1] += knots[1:]
                knots[-1] += xx[end]
                knots *= 0.5
            for idx in np.ndindex(tuple(s)):
                idx = list(idx)
                idx[n] = slice(None)
                #ss = InterpolatedUnivariateSpline(x[n], fnc[idx], bbox=(xx[0],xxe), k=k[n]-1)
                ss = LSQUnivariateSpline(xx, fnc[idx], knots, bbox=(xx[0],xxe), k=kk-1)
                cf[idx] = ss.get_coeffs()
            t[n] = np.pad(knots, kk, 'constant', constant_values=(xx[0],xxe))
            fnc = cf
        ca.append(fnc)
    return t, ca

def print_float_array(o, name, a, prefix):
    print >>o, "%sfloat %s[%d] = {" % (prefix, name, a.size)
    s = "  "
    i = 0
    a = np.where(abs(a) < 100*np.finfo(np.float32).tiny, 0, a)
    for v in a.ravel('C'):
        #o.write(s+repr(float(v)))
        o.write(s + "%.8g" % v)
        s = ","
        i += 1
        if i % 10 == 0:
            print >>o, ","
            s = "  "
    print >>o, "};"
    return a.size * np.float32().nbytes

def print_int_array(o, name, a, prefix):
    print >>o, "%sint %s[%d] = {" % (prefix, name, a.size)
    s = "  "
    i = 0
    for v in a.ravel('C'):
        o.write(s + str(v))
        s = ","
        i += 1
        if i % 10 == 0:
            print >>o, ","
            s = "  "
    print >>o, "};"
    return a.size * np.int32().nbytes

def print_char_array(o, name, a, prefix):
    print >>o, "%sunsigned char %s[%d] = {" % (prefix, name, a.size)
    s = "  "
    i = 0
    for v in a.ravel('C'):
        if v > 255:
            raise ValueError("value > 255 for unsigned char in %s" % name)
        o.write(s + str(v))
        s = ","
        i += 1
        if i % 10 == 0:
            print >>o, ","
            s = "  "
    print >>o, "};"
    align = 4
    return ((a.size + align - 1) // align) * align

def print_intpp_data(o, tag, prefix, func, rgdata, basegrid):
    if prefix:
        prefix += " "
    sz = 0
    int_sz = np.int32().nbytes
    float_sz = np.float32().nbytes
    #ranges = []
    #orderlist = []
    #for r, k in rgdata:
    #    ranges.append(r)
    #    orderlist.append(k)
    spl = TensorSpline(func, rgdata, basegrid)
    coeffs = spl.calc_coeffs()
    for iv, val in enumerate(spl.knot_data):
        n = len([v for v in val if v.used()])
        print >>o, "%sreal x0%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join([str(float(r[0])) for r, v in zip(rgdata, val) if v.used()]))
        print >>o, "%sreal xe%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join([str(float(r[1])) for r, v in zip(rgdata, val) if v.used()]))
        print >>o, "%sreal hi%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join([str(v.inv_h()) for v in val if v.used()]))
        print >>o, "%sint nmap%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join([str(v.count()) for v in val if v.used()]))
        print >>o, "%sint n%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join([str(v.num_coeffs()) for v in val if v.used()]))
        sz += (2*int_sz+2*float_sz) * n;
        #grd = np.mgrid[ranges]
        #grd_shape = grd.shape
        #numpoints = np.product(grd_shape[1:])
        #grd = grd.reshape(grd_shape[0], numpoints, order='F')
        #grd = grd.T
        #fnc = np.array(func(grd, True).T, order='C')
        #fnc.shape = (nvals,)+tuple(reversed(grd_shape[1:]))
        #fnc = fnc.T
        #args = [np.arange(s.step.imag) for s in ranges]
        ti = []
        ai = []
        ci = []
        #idx = [slice(None)]*n+[0]
        #fnca = []
        #for i in range(nvals):
        #    idx[n] = i
        #    fnca.append(fnc[idx])
        #tl, ca = b_ink(args, fnca, orderlist)
        for j in range(len(val)):
            if not val[j].used():
                continue
            t = "t%d_%d%s" % (j, iv, tag)
            ti.append(t)
            sz += print_float_array(o, t, val[j].knots, prefix)
            a = "a%d_%d%s" % (j, iv, tag)
            ai.append(a)
            sz += print_char_array(o, a, val[j].mapping, prefix)
        for i in range(1):
            t = "c%d_%d%s" % (i, iv, tag)
            ci.append(t)
            sz += print_float_array(o, t, coeffs[iv], prefix)
        print >>o, "%sunsigned char *map%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join(ai))
        print >>o, "%sfloat *t%s_%d[%d] = {%s};" % (prefix, tag, iv, n, ", ".join(ti))
        print >>o, "%sfloat *c%s_%d[%d] = {%s};" % (prefix, tag, iv, 1, ", ".join(ci))
    sz += (n + len(basegrid)) * float_sz
    return sz, [[v.order for v in val] for val in spl.knot_data]

################################################################

class PlotManager(object):
    def __init__(self, r, c):
        self.r = r
        self.c = c
        self.i = 1

    def __call__(self):
        pylab.subplot(self.r,self.c,self.i)
        self.i += 1

def check_error_grid2(name, ranges, func1, func2, labels, pm):
    points = np.array(np.mgrid[ranges].T, dtype=np.float32, order="C")
    points = points.view(",".join(['float32']*len(ranges))).reshape(points.shape[:-1])
    def f1(x):
        return tuple(func1(x))
    def f2(x):
        return tuple(func2(x))
    print name
    for i, (l, a1, a2) in enumerate(zip(labels, np.vectorize(f1)(points),np.vectorize(f2)(points))):
        a = abs(a2 - a1)
        print "%s: %g" % (l, max(a.flat))
        if False:
            m = argsort(a, axis=None)
            for i in range(10):
                n = m[-(1+i)]
                idx = unravel_index(n, a.shape)
                print idx, ",".join(["%g" % v for v in points[idx]]), a.flat[n]
        if pm:
            pm()
            pylab.title("%s/%s" % (name,l))
            pylab.hist(a.flat)

def check_error_func2(name, func, funcrange, func0, val, pm):
    l = []
    for i, (x0, h, n) in enumerate(funcrange):
        stop = x0 + (n-1)*h - 1e-6
        l.append(slice(x0+h/2,stop-h/2,(n-1)*1j))
    check_error_grid2(name, l, func, func0, val, pm)
